package simulator;

import java.util.ArrayList;

import app.TheApp;

/**
 * В. Нерретер. Расчёт электрических цепей на персональной ЭВМ.
 *
 * Sinus, Gleich, Dyna, Dyna-NL, Spline, Newton
 */
public final class BasicAnalyzerOriginal {
	private static final String CLASS_NAME = "BasicAnalyzerOriginal";
	//
	/** счётчик для записи результата в массив выходных данных */
	private int mResultIndex = 0;
	//
	/** Dj+1 уравнение 4.4-23 */
	private double dx;
	/** значение предела e (epsilon) для метода Ньютона */
	private double en;
	/** значение предела при решении системы линейных уравнений */
	private double ep;
	/** значение шага */
	private double h;
	/** количество нагрузочных ветвей */
	private int ir;
	/** количество идеальных источников напряжения */
	private int is;
	/** номер интервала */
	private int iv;
	/** количество ветвей */
	private int iz;
	/** коэффициент (флаг) управления для расчета ошибки */
	private boolean jf;
	/** значение порядка k для метода BDF */
	private int kb;
	/** количество узлов без базового */
	private int kn;
	/** количество выдаваемых напряжений ветвей */
	private int ma;
	/** количество выдаваемых токов ветвей */
	private int mb;
	/** количество уравнений системы линейных уравнений */
	private int n;
	/** количество интервалов */
	private int ni;
	/** число уравнений трёхдиагональной системы уравнений */
	private int nt;
	/** количество нелинейных двухполюсников */
	private int nz;
	/** максимально число опорных точек */
	private int nm;
	/** наклон Mk интерполирующего сплайна для Xk */
	private double sk;
	/** Sj+1 уравнение 4.4-23 */
	private double sx;
	/** параметр передачи */
	private double st;
	/** момент времени t */
	private double t;
	/** координата x при интерполяции */
	private double xk;
	/** координата y при интерполяции */
	private double yk;
	//
	/** наличие управляемых источников */
	private boolean ig = false;
	//
	/** емкости ir */
	private double[] C;
	/** проводимость rz */
	private double[] G;
	/** индуктивности ir */
	private double[] L;
	//
	/** матрица узловых проводимостей, матрица коэффициентов n,n */
	private double[][] a;
	/** мнимая часть матрицы узловых проводимостей n,n */
	private double[][] b;
	/** коэффициенты альфа 6 */
	private double[] al;
	/** сплайн коэффициенты a nn,nz */
	private double[][] as;
	/** сплайн коэффициенты b nn,nz */
	private double[][] bs;
	/** сплайн коэффициенты c nn,nz */
	private double[][] cs;
	/** элементы главной диагонали nt */
	private double[] dh;
	/** элементы соседних диагоналей nt */
	private double[] dn;
	/** коэффициенты гамма 6 */
	private double[] ga;
	/** проводимость дискретной токовой модели ir */
	private double[] gd;
	/** мнимая часть проводимости ir */
	private double[] gi;
	/** тип ветви, G(1)/R(2)/C(3)/L(4) ir */
	private char[] ia;
	/** тип временной зависимости K(1)/L(2)/S(3) см. 4.5.2 iz */
	private char[] it;
	/** зависимые узлы is */
	private int[] ka;
	/** массив узлов kn */
	private int[] kf;
	/** независимые узлы is */
	private int[] ku;
	/** начальный узел нагрузочной ветви ir */
	private int[] kv;
	/** конечный узел нагрузочной ветви ir */
	private int[] kz;
	/** управляющие ветви проводимостей, управляемых током nz */
	private int[] mg;
	/** номера ветвей выдаваемых токов ветвей ir */
	private int[] mi;
	/** номера ветвей выдаваемых напряжений ветвей ir */
	private int[] mu;
	/** управляющие ветви управляемых источников тока ir */
	private int[] mz;
	/** число опорных точек нелинейных двухполюсников nz */
	private int[] ns;
	/** мнимая часть напряжений источников is */
	private double[] qi;
	/** напряжения идеальных источников is */
	private double[] qr;
	/** напряжения источников в момент t is */
	private double[] qt;
	/** правая часть трёхдиагональной системы уравнений nt */
	private double[] rs;
	/** токи в резистивных ветвях ir */
	private double[] sa;
	/** мнимые части токов в ветвях ir */
	private double[] sb;
	/** токи источников дискретной токовой модели ir */
	private double[] sd;
	/** коэффициенты e см 5.1.4 ni */
	private double[] se;
	/** коэффициенты управления S или усиления по току B ir */
	private double[] sf;
	/** коэффициенты h см 5.1.4 ni */
	private double[] sh;
	/** мнимые части токов источников ir */
	private double[] si;
	/** токи источников нагрузочных ветвей ir */
	private double[] sr;
	/** стартовое значение nz */
	private double[] sw;
	/** напряжения нагрузочных ветвей ir */
	private double[] ua;
	/** мнимые части напряжений ветвей ir  */
	private double[] ub;
	/** мнимые части узловых потенциалов kn */
	private double[] ui;
	/** узловые потенциалы kn */
	private double[] un;// FIXME проверить! похоже, счет идет с 0!
	/** узловые потенциалы для предыдущего момента времени kn */
	private double[] uv;
	/** отрезки оси и начальные фазовые угла iz */
	private double[] vf;
	/** мнимые части соответствующих напряжений источников ir */
	private double[] vi;
	/** напряжения источников, соответствующие нагрузочным ветвям ir */
	private double[] vr;
	/** правая часть, вектор решения n */
	private double[] x;
	/** опорные точки nn,nz */
	private double[][] xs;
	/** мнимые составляющие правой части или вектора решения n */
	private double[] y;
	/** значение функции в опорной точке nn,nz */
	private double[][] ys;
	/** параметры состояния ir,11 */
	private double[][] zg;
	/** значения параметров двухполюсного накопителя энергии nz,11 */
	private double[][] ze;
	/** наклон или круговая частота см. 4.5.2 iz */
	private double[] zf;
	/** параметры ветвей см. 5.4 nz */
	private double[] zs;
	//
	/**
	 * Оригинальное название: gosub2000, стр. 44 (42)
	 *
	 * Решение системы линейных уравнений
	 *
	 * @return true нет решения
	 */
	private final boolean gosub2000() {
		if( n != 1 ) {
			for( int i = 1; i <= n - 1; i++ ) {
				final double d1 = a[i][i];
				if( Math.abs( d1 ) < ep ) {
					return true;
				}
				for( int j = i + 1; j <= n; j++ ) {
					final double r1 = a[j][i] / d1;
					if( r1 != 0 ) {
						for( int k = i + 1; k <= n; k++ ) {
							a[j][k] = a[j][k] - r1 * a[i][k];
						}
						x[j] = x[j] - r1 * x[i];
					}
				}
			}
		}
		return gosub2030();
	}
	/**
	 * Оригинальное название: завершение gosub2000, стр. 44 (42)
	 *
	 * Решение системы линейных уравнений
	 *
	 * @return true нет решения
	 */
	private final boolean gosub2030() {
		for( int j = n; j >= 1; j-- ) {
			if( Math.abs( a[j][j] ) < ep ) {
				return true;
			}
			double d1 = x[j];
			if( j != n ) {
				for( int k = n; k >= j + 1; k-- ) {
					d1 = d1 - a[j][k] * x[k];
				}
			}
			x[j] = d1 / a[j][j];
		}
		return false;
	}
	/**
	 * Оригинальное название: gosub2100, стр. 29 (27)
	 *
	 * Заполнение массивов узлов kf
	 * Вход:
	 * is - количество идеальных источников напряжения
	 * ka[is] - номер зависимого узла
	 * ku[is] - номер независимого узла
	 *
	 * результат:
	 * kf[kn] - массив узлов
	 */
	private final void gosub2100() {
		kf[0] = 0;// опущено, должно быть
		int j = 0;
	label_i:
		for( int i = 1; i <= kn; i++ ) {
			if( is != 0 ) {
				for( int k = 1; k <= is; k++ ) {
					if( ka[k] == i ) {
						continue label_i;
					}
				}
			}
			j = j + 1;
			kf[i] = j;
		}
		if( is != 0 ) {
			for( int i = 1; i <= is; i++ ) {
				j = ka[i];
				final int k = ku[i];
				kf[j] = kf[k];
			}
		}
	}
	/**
	 * Оригинальное название: gosub2200, стр. 46 (44)
	 *
	 * Решение системы линейных уравнений с симметричной матрицей коэффициентов
	 *
	 * @return true нет решения
	 */
	private final boolean gosub2200() {
		if( n != 1 ) {
			for( int i = 1; i <= n - 1; i++ ) {
				final double d1 = a[i][i];
				if( Math.abs( d1 ) < ep ) {
					return true;
				}
				for( int j = i + 1; j <= n; j++ ) {
					final double r1 = a[i][j] / d1;
					if( r1 != 0 ) {
						for( int k = j; k <= n; k++ ) {
							a[j][k] = a[j][k] - r1 * a[i][k];
						}
						x[j] = x[j] - r1 * x[i];
					}
				}
			}
		}
		return gosub2030();
	}
	/**
	 * Оригинальное название: gosub2300, стр. 43 (41)
	 *
	 * часть aufloes 186 (184), далее есть сокращение, которого нет тут
	 *
	 * Упорядочение системы уравнений
	 *
	 * Вход:
	 * a[n][n] - матрица коэффициентов
	 * ep - предельное значение
	 * n - количество уравнений
	 * x[n] - правая часть
	 *
	 * Результат:
	 * a[n][n] - матрица коэффициентов
	 * x[n] - правая часть
	 *
	 * @return true нет решения
	 */
	private final boolean gosub2300() {
		for( int i = 1; i <= n; i++ ) {
			if( Math.abs( a[i][i] ) <= ep ) {
				int j1 = 0;
				double r1 = 0;
				for( int j = 1; j <= n; j++ ) {
					final double a1 = Math.abs( a[i][j] * a[j][i] );
					if( a1 >= r1 ) {
						r1 = a1;
						j1 = j;
					}
				}
				if( r1 < ep ) {
					return true;
				}
				final double x1 = x[j1];
				x[j1] = x[i];
				x[i] = x1;
				for( int j = 1; j <= n; j++ ) {
					final double a1 = a[j1][j];
					a[j1][j] = a[i][j];
					a[i][j] = a1;
				}
			}
		}
		return false;
	}
	/**
	 * Оригинальное название: gosub2400, стр. 49 (47)
	 *
	 * Ввод управляющих источников тока
	 *
	 * Вход:
	 * i - номер ветви
	 *
	 * Результат:
	 * ig - наличие управляемых источников
	 * mz[i] - номер управляющей ветви
	 * sf[i] - коэффициент управления или усиления по току
	 *
	 * @param i - номер ветви
	 * @param type - управляющий параметр I/U
	 * @param factor - усиление по току/коэффициент управления
	 * @param branch - управляющая ветвь
	 */
	private final void gosub2400(final int i, final char type, final double factor, final int branch) {
		ig = true;
		switch( type ) {
		case 'I':{
			final int ju = -1;
			sf[i] = factor;
			mz[i] = ju * branch;
			break;}
		case 'U':{
			final int ju = 1;
			sf[i] = factor;
			mz[i] = ju * branch;
			break;}
		default:
			ig = false;
			break;
		}
	}
	/**
	 * Оригинальное название: gosub2500, стр. 48 (46)
	 *
	 * einlin 202 (200)
	 *
	 * Ввод элементов цепи для ветви I
	 *
	 * Вход:
	 * i - номер ветви
	 * ir - количество нагрузочных ветвей
	 *
	 * Результат:
	 * gd[i] - проводимость нагрузочной ветви
	 * ig - наличие управляемых источников
	 * ka[il] - зависимые узлы
	 * ku[il] - независимые узлы
	 * kv[i] - начальные узлы нагрузочных ветвей
	 * kz[i] - конечные узлы нагрузочных ветвей
	 * mz[i] - управляющие ветви управляемых источников тока
	 * qt[il] - собственные напряжения идеальных источников напряжения
	 * sd[i] - токи источников
	 * sf[i] - управляющие коэффициенты или усиление по току
	 *
	 * для i > ir напряжение источника
	 * ka[i],ku[i],qt[il]
	 *
	 * @param i - номер ветви. если i > ir, вводится напряжение
	 * @param j - из узла
	 * @param k - к узлу
	 * @param type - тип узла R/G/I/S/N
	 * @param value - номинал узла, сопротивление/проводимость/ток/индуктивность или напряжение
	 * @param ctype управляющий параметр I/U
	 * @param factor усиление по току/коэффициент управления
	 * @param branch управляющая ветвь
	 */
	private final void gosub2500(final int i,
			final int j, final int k,
			final char type, final double value,
			final char ctype, final double factor, final int branch
			)
	{
		if( i <= ir ) {
			// в оригинале это продолжение ввода данных для этой же ветви
			if( kv[i] != j || kz[i] != k ) {
				kv[i] = j;
				kz[i] = k;
				gd[i] = 0;
				sd[i] = 0;
				mz[i] = 0;
			}
			switch( type ) {
			case 'R':
				gd[i] = 1. / value;
				break;
			case 'G':
				gd[i] = value;
				break;
			case 'I':
				sd[i] = value;
				break;
			case 'S':
				gosub2400( i, ctype, factor, branch );
				break;
			case 'N':
				return;
			default:
				System.out.println( CLASS_NAME + ".gosub2500 incorrect input" );
				return;
			}
			return;
		}
		final int il = i - ir;
		ka[il] = j;
		ku[il] = k;
		qt[il] = value;
	}
	/**
	 * Оригинальное название: gosub2600, стр. 50 (48)
	 *
	 * vorver 185 (183)
	 *
	 * Перемещение идеальных источников напряжения
	 *
	 * Вход:
	 * ir, is, ka[i], ku[ir], kz[ir], qt[is]...
	 *
	 * Результат:
	 * vr[ir] - напряжения источников, соответствующих нагрузочным ветвям
	 *
	 */
	private final void gosub2600() {
		for( int i = 1; i <= ir; i++ ) {
			vr[i] = 0;
			if( is != 0 ) {
				for( int il = 1; il <= is; il++ ) {
					if( kv[i] == ka[il] ) {
						vr[i] = vr[i] + qt[il];
					}
					if( kz[i] == ka[il] ) {
						vr[i] = vr[i] - qt[il];
					}
				}
			}
		}
	}
	/**
	 * Оригинальное название: gosub2700, стр. 53 (51)
	 *
	 * aufloes 186 (184)
	 *
	 * Составление и решение системы линейных уравнений
	 *
	 * Вход:
	 * gd[ir], ig, ir, kf[kn], kv[ir], kz[ir], mz[ir], sd[ir], sf[ir], vr[ir]
	 *
	 * Результат:
	 * x[n] - вектор решения
	 *
	 * @return true - решение невозможно
	 */
	private final boolean gosub2700() {
		for( int i = 1; i <= n; i++ ) {
			x[i] = 0;
			for( int j = 1; j <= n; j++ ) {
				a[i][j] = 0;
			}
		}
		for( int i = 1; i <= ir; i++ ) {
			double s1 = sd[i] + gd[i] * vr[i];
			final int j = kv[i];
			final int k = kz[i];
			final int jr = kf[j];
			final int kr = kf[k];
			if( jr != kr ) {// 2728 -> 2774
				a[jr][jr] = a[jr][jr] + gd[i];
				a[jr][kr] = a[jr][kr] - gd[i];
				a[kr][jr] = a[kr][jr] - gd[i];
				a[kr][kr] = a[kr][kr] + gd[i];
				if( mz[i] != 0 ) {// 2738 -> 2770
					final int ms = Math.abs( mz[i] );
					double g1;
					if( mz[i] >= 0 ) {// 2742 -> 2750
						g1 = sf[i];
						s1 = s1 + sf[i] * vr[ms];
					} else {
						g1 = sf[i] * gd[ms];
						s1 = s1 + g1 * vr[ms] + sf[i] * sd[ms];
					}
					int js = kv[ms];
					int ks = kz[ms];
					js = kf[js];
					ks = kf[ks];
					a[jr][js] = a[jr][js] + g1;
					a[jr][ks] = a[jr][ks] - g1;
					a[kr][js] = a[kr][js] - g1;
					a[kr][ks] = a[kr][ks] + g1;
				}
				x[jr] = x[jr] - s1;
				x[kr] = x[kr] + s1;
			}
		}
		ep = 1e-12;// FIXME check ep value;
		if( ig ) {
			if( gosub2300() ) {
				return true;
			}
			return gosub2000();
		}
		return gosub2200();
	}
	/**
	 * Оригинальное название: gosub2800, стр. 54 (52)
	 *
	 * Расчёт узловых напряжений
	 *
	 * Вход:
	 * is, ka[is],kf[kn],ku[is],qt[is],x[n]
	 *
	 * Результат:
	 * un[kn] - узловые напряжения
	 */
	private final void gosub2800() {
		for( int j = 1; j <= kn; j++ ) {
			final int k = kf[j];
			un[j] = x[k];
		}
		un[0] = 0;// FIXME un[ 0 ] 0000!
		if( is != 0 ) {
			for( int il = 1; il <= is; il++ ) {
				final int j = ka[il];
				final int k = ku[il];
				un[j] = un[k] + qt[il];
			}
		}
	}
	/**
	 * Оригинальное название: gosub2900, стр. 55 (53)
	 *
	 * zweig 192 (190)
	 * Расчёт параметров ветвей
	 *
	 * Вход:
	 * gd[ir], ir, kf[kn], kv[ir], kz[ir], mz[ir], sd[ir], un[kn]
	 *
	 * Результат:
	 * sa[ir] - токи нагрузочных ветвей
	 * ua[ir] - напряжения нагрузочных ветвей
	 */
	private final void gosub2900() {
		for( int i = 1; i <= ir; i++ ) {
			final int j = kv[i];
			final int k = kz[i];
			ua[i] = un[j] - un[k];
			sa[i] = ua[i] * gd[i] + sd[i];
		}
		//if( zweigart[i] == 'S' ) {
		for( int i = 1; i <= ir; i++ ) {
			final int ms = Math.abs( mz[i] );
			if( ms != 0 ) {
				double s1;
				if( mz[i] < 0 ) {
					s1 = sf[i] * sa[ms];
				} else {
					s1 = sf[i] * ua[ms];
				}
				sa[i] = sa[i] + s1;
			}
		}
		//}
	}
	/**
	 * Оригинальное название: gosub3000, стр. 71 (73)
	 *
	 * kaufloes 190 (188) сокращение системы комплексных уравнений
	 *
	 * Решение системы линейных уравнений
	 *
	 * Вход:
	 * a[n][n] - действительная часть матрицы коэффициентов
	 * b[n][n] - мнимая часть матрицы коэффициентов
	 * ep - предельное значение e (epsilon)
	 * ig - наличие управляемых источников тока
	 * n - количество уравнений
	 * x[n] - действительная составляющая правой части
	 * y[n] - мнимая составляющая правой части
	 *
	 * Результат
	 * x[n] - действительная часть вектора решения
	 * y[n] - мнимая часть вектора решения
	 *
	 * @return true - решение невозможно
	 */
	private final boolean gosub3000() {
		final double e2 = ep * ep;
		if( n != 1 ) {
			for( int i = 1; i <= n - 1; i++ ) {
				final double d1 = a[i][i];
				final double d2 = b[i][i];
				final double d3 = d1 * d1 + d2 * d2;
				if( d3 < e2 ) {
					System.out.println( CLASS_NAME + ".gosub3000 solving impossible" );
					return true;
				}
				for( int j = i + 1; j <= n; j++ ) {
					final double r1 = (a[j][i] * d1 + b[j][i] * d2) / d3;
					final double r2 = (b[j][i] * d1 - a[j][i] * d2) / d3;
					if( (Math.abs( r1 ) + Math.abs( r2 )) != 0 ) {
						for( int k = i + 1; k <= n; k++ ) {
							a[j][k] = a[j][k] - r1 * a[i][k] + r2 * b[i][k];
							b[j][k] = b[j][k] - r1 * b[i][k] - r2 * a[i][k];
						}
						x[j] = x[j] - r1 * x[i] + r2 * y[i];
						y[j] = y[j] - r1 * y[i] - r2 * x[i];
					}
				}
			}
		}
		return gosub3042();
	}
	/**
	 * Завершающая часть gosub3000, вызывается и из других мест
	 *
	 * kaufloes 191 (181)
	 *
	 * @return true решение невозможно
	 */
	private final boolean gosub3042() {
		final double e2 = ep * ep;
		// обратная подстановка
		for( int j = n; j >= 1; j-- ) {
			double d1 = x[j];
			double d2 = y[j];
			if( j != n ) {
				for( int k = n; k >= j + 1; k-- ) {
					d1 = d1 - a[j][k] * x[k] + b[j][k] * y[k];
					d2 = d2 - b[j][k] * x[k] - a[j][k] * y[k];
				}
			}
			final double r1 = a[j][j];
			final double r2 = b[j][j];
			final double r3 = r1 * r1 + r2 * r2;
			if( r3 < e2 ) {
				System.out.println( CLASS_NAME + ".gosub3000 solving impossible" );
				return true;
			}
			x[j] = (d1 * r1 + d2 * r2) / r3;
			y[j] = (d2 * r1 - d1 * r2) / r3;
		}
		return false;
	}
	/**
	 * Оригинальное название: gosub3100, стр. 67 (65)
	 *
	 * Вход:
	 * ar - действительная часть комплексного числа
	 * bi - мнимая часть комплексного числа
	 * pi - число
	 *
	 * Результат:
	 * ph - угол комплексного числа
	 * rp - модуль комплексного числа
	 *
	 * Преобразование координат R->P
	 */
	/*private final boolean gosub3100() {
		final double rp = Math.hypot( ar, bi );
		ph = 90 * Math.signum( bi );
		if( ar != 0 ) {
			ph = 90. * (1 - Math.signum( ar )) * Math.signum( ph + 1. );
			ph = 180. * Math.ath( ni / ar ) / pi + ph;
		}
	}*/
	/**
	 * Оригинальное название: gosub3200, стр. 73 (71)
	 *
	 * kaufloes 190 (188)
	 *
	 * Решение системы линейных уравнений с симметричной матрицей коэффициентов
	 *
	 * @return true - решение невозможно
	 */
	private final boolean gosub3200() {
		final double e2 = ep * ep;
		if( n != 1 ) {
			for( int i = 1; i <= n - 1; i++ ) {
				final double d1 = a[i][i];
				final double d2 = b[i][i];
				final double d3 = d1 * d1 + d2 * d2;
				if( d3 < e2 ) {
					return true;
				}
				for( int j = i + 1; j <= n; j++ ) {
					final double r1 = (a[i][j] * d1 + b[i][j] * d2) / d3;
					final double r2 = (b[i][j] * d1 - a[i][j] * d2) / d3;
					if( (Math.abs( r1 ) + Math.abs( r2 )) != 0 ) {
						for( int k = j; k <= n; k++ ) {
							a[j][k] = a[j][k] - r1 * a[i][k] + r2 * b[i][k];
							b[j][k] = b[j][k] - r1 * b[i][k] - r2 * a[i][k];
						}
						x[j] = x[j] - r1 * x[i] + r2 * y[i];
						y[j] = y[j] - r1 * y[i] - r2 * x[i];
					}
				}
			}
		}
		return gosub3042();
	}
	/**
	 * Оригинальное название: gosub3300, стр. 72 (70)
	 *
	 * kaufloes 189 (187)
	 *
	 * Упорядочение системы уравнений
	 *
	 * Вход:
	 * a[n][n] - матрица коэффициентов
	 * ep - предельное значение e (epsilon)
	 * n - количество уравнений
	 * x[n] - правая часть
	 *
	 * Результат:
	 * a[n][n] - матрица коэффициентов
	 * x[n] - правая часть
	 *
	 * @return true - решение невозможно
	 */
	private final boolean gosub3300() {
		for( int i = 1; i <= n; i++ ) {
			double b1 = Math.hypot( a[i][i], b[i][i] );// sqrt a^2 + b^2
			if( b1 <= ep ) {// FIXME kaufloes ep^2
				int j1 = 0;
				double r1 = 0.;
				for( int j = 1; j <= n; j++ ) {
					b1 = Math.hypot( a[i][j], b[i][j] );// sqrt a^2 + b^2
					final double b2 = Math.hypot( a[j][i], b[j][i] );// sqrt a^2 + b^2
					final double a1 = b1 * b2;
					if( a1 >= r1 ) {// FIXME kaufloes a1 > r1
						r1 = a1;
						j1 = j;
					}
				}
				if( r1 < ep ) {
					return true;
				}
				final double x1 = x[j1];
				final double y1 = y[j1];
				x[j1] = x[i];
				y[j1] = y[i];
				x[i] = x1;
				y[i] = y1;
				for( int j = 1; j <= n; j++ ) {
					final double a1 = a[j1][j];
					b1 = b[j1][j];
					a[j1][j] = a[i][j];
					b[j1][j] = b[i][j];
					a[i][j] = a1;
					b[i][j] = b1;
				}
			}
		}
		return false;
	}
	/**
	 * Оригинальное название: gosub3400, стр. 76 (74)
	 *
	 * Определение выводимых величин
	 *
	 * ir - количество нагрузочных ветвей
	 *
	 * результат:
	 * ma - количество выводимых напряжений ветвей
	 * mb - количество выводимых токов ветвей
	 * mi[ir] - номера ветвей выводимых токов
	 * mu[ir] - номера ветвей выводимых напряжений
	 *
	 * @param vbranches номера ветвей напряжений
	 * @param ibranches номера ветвей токов
	 */
	private final void gosub3400(final int[] vbranches, final int[] ibranches) {// sinus
		for( int i = 1; i <= ir; i++ ) {
			mi[i] = i;
			mu[i] = i;
		}
		// кол-во напряжений ветвей 1
		ma = vbranches.length;
		for( int i = 1; i <= ma; i++ ) {
			mu[i] = vbranches[i - 1];
		}
		// кол-во токов ветвей 0
		mb = ibranches.length;
		for( int i = 1; i <= mb; i++ ) {
			mi[i] = ibranches[i - 1];
		}
	}
	/**
	 * Оригинальное название: gosub3500, стр. 75 (73)
	 *
	 * Ввод элементов цепи
	 *
	 * i - номер ветви
	 * ir - количество нагрузочных ветвей
	 *
	 * результат:
	 * g[i], gd[i], ig, ka[il], ku[il], kv[i], kz[i], mz[i], qt[il], qi[il], sd[i], si[i], sf[i]
	 *
	 * @param i номер ветви. при i > ir вводится напряжение
	 * @param j от узла
	 * @param k к узлу
	 * @param type элемент R/G/L/C/I/S/N
	 * @param value значение R/G/L/C/I/S/N
	 * @param im мнимая часть тока/напряжения
	 * @param ctype управляющий параметр I/U
	 * @param factor усиление по току/коэффициент управления
	 * @param branch управляющая ветвь
	 */
	private final void gosub3500(final int i,
		final int j, final int k,
		final char type, final double value, final double im,
		final char ctype, final double factor, final int branch
			)
	{
		if( i <= ir ) {
			kv[i] = j;
			kz[i] = k;
			gd[i] = 0;
			L[i] = 0;
			C[i] = 0;
			sd[i] = 0;
			si[i] = 0;
			mz[i] = 0;
			switch( type ) {
			case 'R':
				gd[i] = 1. / value;
				break;
			case 'G':
				gd[i] = value;
				break;
			case 'L':
				L[i] = value;
				break;
			case 'C':
				C[i] = value;
				break;
			case 'I':
				sd[i] = value;
				si[i] = im;
				break;
			case 'S':
				gosub2400( i, ctype, factor, branch );
				break;
			case 'N':
			default:
				return;
			}
			return;
		}
		final int il = i - ir;
		ka[il] = j;
		ku[il] = k;
		qt[il] = value;
		qi[il] = im;
	}
	/**
	 * Оригинальное название: gosub3600, стр. 77 (75)
	 *
	 * kvorver 185 (183)
	 *
	 * Перемещение идеальных источников напряжения
	 * @param ir, is, ka[is], kv[ir], kz[ir], qi[is], qt[is]
	 * @return vi[ir] - мнимая часть
	 * @return vr[ir] - действительная часть напряжения источника, соответствующего нагрузочной ветви
	 */
	private final void gosub3600() {
		/*for( int i = 1; i <= ir; i++ ) {
			vr[i] = 0;
			vi[i] = 0;
			if( is != 0 ) {
				for( int il = 1; il <= is; il++ ) {
					double jv = 1.;
					if( kv[i] != ka[il] ) {
						if( kz[i] < ka[il] ) {
							jv = -1.;
						} else {
							continue;
						}
					}
					vr[i] = vr[i] + jv * qt[il];
					vi[i] = vi[i] + jv * qi[il];
				}
			}
		}*/
		// kvorver
		for( int i = 1; i <= ir; i++ ) {
			vr[i] = 0;
			vi[i] = 0;
			if( is != 0 ) {
				for( int il = 1; il <= is; il++ ) {
					if( kv[i] == ka[il] ) {
						vr[i] = vr[i] + qt[il];// pascal: qr
						vi[i] = vi[i] + qi[il];
					}
					if( kz[i] == ka[il] ) {
						vr[i] = vr[i] - qt[il];// pascal: qr
						vi[i] = vi[i] - qi[il];
					}
				}
			}
		}
	}
	/**
	 * Оригинальное название: gosub3800, стр. 81 (79)
	 *
	 * kknoten 191 (189) расчет комплексных узловых напряжений
	 *
	 * Вычисление узловых напряжений
	 *
	 * Вход:
	 * is
	 * ka[is],kf[is],kn, ku[is],qi[is],qt[is],x[n],y[n]
	 *
	 * Результат:
	 * ui[kn] - мнимая часть узлового напряжения
	 * un[kn] - действительная част узлового напряжения
	 */
	private final void gosub3800() {
		for( int j = 1; j <= kn; j++ ) {
			final int k = kf[j];
			un[j] = x[k];
			ui[j] = y[k];
		}
		un[0] = 0;
		ui[0] = 0;
		if( is != 0 ) {
			for( int il = 1; il <= is; il++ ) {
				final int j = ka[il];
				final int k = ku[il];
				un[j] = un[k] + qt[il];
				ui[j] = ui[k] + qi[il];
			}
		}
	}
	/**
	 * Оригинальное название: gosub3900, стр. 82 (80)
	 *
	 * kzweig 192 (190) расчет комплексных параметров ветви
	 *
	 * Вычисление параметров ветвей
	 *
	 * gd[ir], sd[ir], un[kn]
	 * s1
	 * sa[ir], ua[ir]
	 */
	private final void gosub3900() {
		for( int i = 1; i <= ir; i++ ) {
			final int j = kv[i];
			final int k = kz[i];
			ua[i] = un[j] - un[k];
			ub[i] = ui[j] - ui[k];
			sa[i] = gd[i] * ua[i] - gi[i] * ub[i] + sd[i];
			sb[i] = gi[i] * ua[i] + gd[i] * ub[i] + si[i];
		}
		for( int i = 1; i <= ir; i++ ) {
			final int ms = Math.abs( mz[i] );
			if( ms != 0 ) {
				double s1;
				double s2;
				if( mz[i] >= 0 ) {
					s1 = sf[i] * ua[ms];
					s2 = sf[i] * ub[ms];
				} else {
					s1 = sf[i] * sa[ms];
					s2 = sf[i] * sb[ms];
				}
				sa[i] = sa[i] + s1;
				sb[i] = sb[i] + s2;
			}
		}
		/*
		// kzweig:
		for( int i = 1; i <= ir; i++ ) {
			final int j = kv[i];
			final int k = kz[i];
			ua[i] = un[j] - un[k];
			ub[i] = ui[j] - ui[k];
			sa[i] = gd[i] * ua[i] - gi[i] * ub[i] + sd[i];
			sb[i] = gi[i] * ua[i] + gd[i] * ub[i] + si[i];
			//
			final int ms = mz[i];
			if( ms > 0 ) {
				sa[i] = sf[i] * ua[ms];
				sb[i] = sf[i] * ub[ms];
			} else {
				sa[i] = sf[i] * sa[-ms];
				sb[i] = sf[i] * sb[-ms];
			}
		}
		for( int i = 1; i <= is; i++ ) {
			int j = i + ir;
			ua[j] = qt[i];// kzweig qr
			ub[j] = qi[i];
			sa[j] = 0;
			sb[j] = 0;
			for( int k = 1; k <= ir; k++ ) {
				if( ka[j] == kv[k] ) {
					sa[j] = sa[j] - sa[k];
					sb[j] = sb[j] - sb[k];
				}
				if( ka[j] == kz[k] ) {
					sa[j] = sa[j] + sa[k];
					sb[j] = sb[j] + sb[k];
				}
			}
		}
		*/
	}
	/**
	 * Оригинальное название: gosub4000, стр. 114 (112)
	 *
	 * dismod 198 (196)
	 *
	 * Расчет дискретной токовой модели для каждого линейного двухполюсника
	 *
	 * Вход:
	 * al[kb], C[i], G[i], h, ia[i], ir, is, it[i], ka[is], kb
	 * kv[i], kz[i], L[i], nz, qr[is], sr[i], t, vf[i], zf[i]
	 * zg[i,11]
	 *
	 * Результат:
	 * gd[ir], qt[is], sd[ir], vr[ir]
	 *
	 */
	private final void gosub4000() {
		if( is != 0 ) {
			for( int il = 1; il <= is; il++ ) {
				final int i = il + ir;
				st = qr[il];
				gosub4400( i );
				qt[il] = st;
			}
		}
		gosub2600();
		for( int i = nz + 1; i <= ir; i++ ) {
			// FIXME ? gd[i] = 0; sd[i] = 0;
			if( ia[i] == 'G'// 1 нелинейная проводимость
				|| ia[i] == 'R' ) {// 2 нелинейное сопротивление
				gd[i] = G[i];
				st = sr[i];
				gosub4400( i );
				sd[i] = st;
			} else {
				if( ia[i] == 'C' ) {// 3 нелинейная ёмкость
					gd[i] = -al[0] * C[i] / h;// FIXME al[0] index 0!
					double s1 = 0;
					for( int j = 1; j <= kb; j++ ) {
						s1 = s1 + al[j] * zg[i][j];
					}
					sd[i] = -C[i] * s1 / h;// FIXME al[0] index 0!
				} else if( ia[i] == 'L' ) {// 4 нелинейная индуктивность
					gd[i] = -h / (al[0] * L[i]);
					double s1 = 0;
					for( int j = 1; j <= kb; j++ ) {
						s1 = s1 + al[j] * zg[i][j];
					}
					sd[i] = -s1 / al[0];
				}
				/* внесено внутрь предыдущих условий
				double s1 = 0;
				for( int j = 1; j <= kb; j++ ) {
					s1 = s1 + al[j] * zg[i][j];
				}
				switch( ia[i] ) {// первых 2х тут не будет
				case 'G':// 1 нелинейная проводимость
				case 'R':// 2 нелинейное сопротивление
					break;
				case 'C':// 3 нелинейная ёмкость
					sd[i] = -C[i] * s1 / h;
					break;
				case 'L':// 4 нелинейная индуктивность
					sd[i] = -s1 / al[0];
					break;
				}*/
			}
		}
	}
	/**
	 * Оригинальное название: gosub4100, стр. 115 (113)
	 *
	 * Вычисление параметров ветви
	 *
	 * Вход:
	 * kn, un[kn]
	 *
	 * Результат:
	 * dx, sa[ir], sx, ua[ir], un[kn]
	 *
	 * @return true - решение невозможно
	 */
	private final boolean gosub4100() {// dyna-lin
		for( int i = 1; i <= kn; i++ ) {
			uv[i] = un[i];
		}
		if( gosub2700() ) {
			return true;
		}
		gosub2800();
		if( jf ) {// != 0 end of anazeit 200 (198)
			for( int i = 1; i <= kn; i++ ) {
				dx = dx + Math.abs( un[i] - uv[i] );
				sx = sx + Math.abs( un[i] );
			}
		}
		gosub2900();
		return false;
	}
	/**
	 * Оригинальное название: gosub4300, стр. 115 (113)
	 *
	 * Запоминание параметров состояния
	 *
	 * Вход:
	 * ia[ir], ir, ma, mb, mi[ir], mu[ir], sa[ir], t, ua[ir], zg[ir,11]
	 *
	 * Результат:
	 * zg[ir][11] - вывод параметров ветви
	 *
	 */
	private final void gosub4300(final double[][] result) {// dyna-lin
		for( int i = 1; i <= ir; i++ ) {
			for( int j = 11; j >= 2; j-- ) {// FIXME why 11 to 2?
				zg[i][j] = zg[i][j - 1];
			}
			switch( ia[i] ) {
			case 'G':// 1 нелинейная проводимость
			case 'C':// 3 нелинейная ёмкость
				zg[i][1] = ua[i];
				break;
			case 'R':// 2 нелинейное сопротивление
			case 'L':// 4 нелинейная индуктивность
				zg[i][1] = sa[i];
				break;
			}
		}
		System.out.println( CLASS_NAME + ".gosub4300 t = " + t + " s " );
		if( ma >= 1 ) {
			for( int i = 1; i <= ma; i++ ) {
				final int j = mu[i];
				if( j <= ir ) {
					result[i - 1][mResultIndex] = ua[j];
					System.out.println( CLASS_NAME + ".gosub4300 U" + j + " = " + ua[j] + " V " );
				}
			}
		}
		if( mb >= 1 ) {
			for( int i = 1; i <= mb; i++ ) {
				final int j = mu[i];
				if( j <= ir ) {
					result[i - 1 + ma][mResultIndex] = sa[j];
					System.out.println( CLASS_NAME + ".gosub4300 I" + j + " = " + sa[j] + " A " );
				}
			}
		}
	}
	/**
	 * Оригинальное название: часть gosub4300, стр. 115 (113)
	 *
	 * Запоминание параметров состояния
	 *
	 * Вход:
	 * ia[ir], ir, ma, mb, mi[ir], mu[ir], sa[ir], t, ua[ir], zg[ir,11]
	 *
	 * Результат:
	 * вывод параметров ветви
	 *
	 */
	private final void gosub4326(final double[][] result) {
		if( ma >= 1 ) {
			for( int i = 1; i <= ma; i++ ) {
				final int j = mu[i];
				if( j <= ir ) {
					//result[i - 1][mResultIndex] = ua[j];
					TheApp.log( CLASS_NAME + ".gosub4300 U" + j + " = " + ua[j] + " V " );
				}
			}
		}
		if( mb >= 1 ) {
			for( int i = 1; i <= mb; i++ ) {
				final int j = mu[i];
				if( j <= ir ) {
					//result[i - 1 + ma][mResultIndex] = sa[j];
					TheApp.log( CLASS_NAME + ".gosub4300 I" + j + " = " + sa[j] + " A " );
				}
			}
		}
	}
	/**
	 * Оригинальное название: gosub4400, стр. 112 (110)
	 *
	 * augwert 198 (196)
	 *
	 * Расчет мгновенных значений параметра источника
	 *
	 * Вход:
	 * i - номер ветви
	 * it[i] - тип временной зависимости
	 * st - максимальное значение
	 * t - момент времени t
	 * vf[i] - отрезок оси или начальный фазовый угол
	 * zf[i] - наклон или круговая частота
	 *
	 * Результат:
	 * st - мгновенные значения параметра источника
	 *
	 */
	private final void gosub4400(final int i) {// dyna-lin
		if( st != 0 ) {
			double s1 = 0;
			switch( it[i] ) {
			case 'K':// 1:// K
				s1 = st;
				break;
			case 'L':// 2:// L
				s1 = zf[i] * t + vf[i];
				if( Math.abs( s1 ) > Math.abs( st ) ) {
					s1 = st;
				}
				break;
			case 'S':// 3:// S
				s1 = st * Math.cos( zf[i] * t + vf[i] );
				break;
			}
			st = s1;
		}
	}
	/**
	 * Оригинальное название: gosub4500, стр. 110 (108)
	 *
	 * einlin стр. 202 (200)
	 *
	 * Ввод элементов цепи
	 *
	 * Вход:
	 * i - номер ветви
	 * ir - количество нагрузочных ветвей
	 *
	 * Результат:
	 * c[i],g[i],ia[],ig,it,[i],kv[i],kz[i],L[i],mz[i],
	 * sf[i],sr[i],vf[i],zf[i],zg[i][1]...
	 *
	 * для i > ir
	 * it[i],ka[i],ku[i],qr[i],vf[i],zf[i]
	 *
	 * @param i - номер ветви. если i > ir, вводится напряжение
	 * @param j - из узла
	 * @param k - к узлу
	 * @param type - тип узла R/G/C/L
	 * @param value - номинал узла, сопротивление/проводимость/емкость/индуктивность или напряжение
	 * @param start - ток для параметра I для сопротивления/проводимости или
	 * начальное значение емкости/индуктивности
	 * @param source - тип источника для сопротивления/проводимости I/S/N
	 * @param stype - зависимость от времени, K/L/S
	 * @param v1 для L - временной коэффициент, для S - круговая частота
	 * @param v2 для L - отрезок оси, для S - начальный фазовый угол
	 * @param ctype управляющий параметр I/U
	 * @param factor усиление по току/коэффициент управления
	 * @param branch управляющая ветвь
	 */
	private final void gosub4500(final int i,
		final int j, final int k,
		final char type, double value, final double start,
		final char source,
		final char stype, final double v1, final double v2,
		final char ctype, final double factor, final int branch
		)
	{// dyna-lin
		if( i <= ir ) {
			kv[i] = j;// из узла
			kz[i] = k;// к узлу
			G[i] = 0;
			L[i] = 0;
			C[i] = 0;
			// ?
			//sr[i] = 0;
			//si[i] = 0;
			//mz[i] = 0;
			// R/G/L/C/I/S/T
			switch( type ) {
			case 'R':
				value = 1. / value;
			case 'G':
				G[i] = value;
				ia[i] = 'G';
				sr[i] = 0.;
				mz[i] = 0;
				switch( source ) {
				case 'I':
					sr[i] = start;
					gosub4600( i, stype, v1, v2 );
					break;
				case 'S':
					gosub2400( i, ctype, factor, branch );
					break;
				case 'N':
					return;
				}
				break;
			case 'C':
				ia[i] = 'C';
				C[i] = value;
				zg[i][1] = start;
				break;
			case 'L':
				ia[i] = 'L';
				L[i] = value;
				zg[i][1] = start;
				break;
			}
			return;
		}
		final int il = i - ir;
		ka[il] = j;
		ku[il] = k;
		qr[il] = value;// напряжение источника
		gosub4600( i, stype, v1, v2 );
	}
	/**
	 * Оригинальное название: gosub4600, стр. 111 (109)
	 *
	 * Ввод временной зависимости
	 *
	 * Вход:
	 * i - номер ветви
	 *
	 * Результат:
	 * it[i] - тип временной зависимости
	 * vf[i] - начальный коэффициент (отрезок оси или начальный фазовый угол)
	 * zf[i] - временной коэффициент (крутизна или круговая частота)
	 *
	 * @param i
	 * @param type зависимость от времени, K/L/S
	 * @param v1 для L - временной коэффициент, для S - круговая частота
	 * @param v2 для L - отрезок оси, для S - начальный фазовый угол
	 */
	private final void gosub4600(final int i,
		final char type, final double v1, final double v2
		)
	{// dyna-lin
		switch( type ) {
		case 'K':
			it[i] = 'K';// 1
			break;
		case 'L':
			it[i] = 'L';// 2
			zf[i] = v1;
			vf[i] = v2;
			break;
		case 'S':
			it[i] = 'S';// 3
			zf[i] = v1;
			vf[i] = v2;
			break;
		}
	}
	/**
	 * Оригинальное название: gosub4800, стр. 103 (101)
	 *
	 * koeff 198 (196)
	 *
	 * Вычисление коэффициентов
	 *
	 * Вход:
	 * kb - значение порядка k BDF метода
	 *
	 * Результат:
	 * al[i] - коэффициенты Ai (alfa) для 0 <= i <= k
	 * vf[i] - числитель для Yi (gamma) для 1 <= i <= k
	 */
	private final void gosub4800() {
		al[0] = -kb;
		al[1] = kb;
		ga[1] = kb;
		if( kb >= 2 ) {
			for( int i2 = 2; i2 <= kb; i2++ ) {
				final int i1 = i2 - 1;
				ga[i2] = ga[i1] * (i1 - kb) / i2;
				al[i2] = ga[i2] / i2;
				al[0] = al[0] - al[i2];
			}
		}
	}
	/**
	 * Оригинальное название: gosub5006, стр. 131 (129)
	 *
	 * tridiag 194 (192)
	 *
	 * Решение трёхдиагональной системы уравнений
	 *
	 * Вход:
	 * nt - количество уравнений
	 * dh[nt] - элементы главной диагонали
	 * dn[nt] - элементы дополнительной диагонали
	 * rs[nt] - правая часть
	 *
	 * Результат:
	 * rs[nt] - вектор решения
	 *
	 * @param true - граничное условие б), false - а).
	 * аналог randbed[i]='B'
	 * @return true - решение невозможно
	 */
	private final boolean gosub5006(final boolean isConditionB) {// редукция
		if( nt <= 1 ) {
			return true;
		}
		/*
		for( int i2 = 2; i2 <= nt; i2++ ) {
			final int i1 = i2 - 1;
			final double r1 = dn[i1] / dh[i1];
			dh[i2] = dh[i2] - r1 * dn[i1];
			rs[i2] = rs[i2] - r1 * rs[i1];
		}
		*/
		/* при решении системы 5.1-34 слудет учесть, что матрица коэффициентов не симметрична:
		 * элемент дополнительной диагонали в строке Nt в 2 раза больше элемента в строке Nt-1 и столбце Nt
		 * той же диагонали.
		 * Поэтому редукцию элемента главной диагонали и элемента правой части в строке Nt после вызова
		 * подпрограммы 5000 необходимо выплонить ещё раз.
		 */
		/*
		if( isConditionB ) {
			dh[nt] = dh[nt] - r1 * dn[nt - 1];
			rs[nt] = rs[nt] - r1 * rs[nt - 1];
		}
		*/
		//
		int i2 = 2;
		for( ; i2 < nt; i2++ ) {
			final int i1 = i2 - 1;
			final double r1 = dn[i1] / dh[i1];// коэффициент редукции
			dh[i2] = dh[i2] - r1 * dn[i1];
			rs[i2] = rs[i2] - r1 * rs[i1];
		}
		// i2 = nt;// уже равно
		final int i1 = i2 - 1;
		final double r1 = dn[i1] / dh[i1];
		dh[i2] = dh[i2] - r1 * dn[i1];
		rs[i2] = rs[i2] - r1 * rs[i1];
		if( isConditionB ) {
			/* при решении системы 5.1-34 следует учесть, что матрица коэффициентов не симметрична:
			 * элемент дополнительной диагонали в строке Nt в 2 раза больше элемента в строке Nt-1 и столбце Nt
			 * той же диагонали.
			 * Поэтому редукцию элемента главной диагонали и элемента правой части в строке Nt после вызова
			 * подпрограммы 5000 необходимо выполнить ещё раз.
			 */
			dh[nt] = dh[nt] - r1 * dn[nt - 1];
			rs[nt] = rs[nt] - r1 * rs[nt - 1];
		}
		return false;
	}
	private final void gosub5026() {// обратная подстановка
		rs[nt] = rs[nt] / dh[nt];
		for( int i1 = nt - 1; i1 >= 1; i1-- ) {
			rs[i1] = (rs[i1] - dn[i1] * rs[i1 + 1]) / dh[i1];
		}
	}
	/**
	 * Оригинальное название: gosub5100, стр. 132 (130)
	 *
	 * splines 195 (193)
	 *
	 * Вычисление сплайн-коэффициентов согласно граничному условию а)
	 * A) - через обе граничные точки кривая должна проходить с нулевой кривизной, "превращаясь" в прямую.
	 * это означает, что вторая производная любого интерполирующего сплайна на границах области равна нулю
	 * стр. 126 (124), уравнения 5.1-8
	 *
	 * Вход:
	 * i - номер ветви
	 * ni - количество N интервалов
	 * xs[j][i] - опорные точки Xj
	 * ys[j][i] - значения функции Yj для опорных точек
	 *
	 * Используются:
	 * dh[ni] - элементы главной диагонали
	 * dn[ni] - элементы дополнительной диагонали
	 * nt - количество уравнений
	 * rs[ni] - правая часть (вектор решения)
	 * se[ni] - коэффициенты Ej
	 * sh[ni] - коэффициенты Hj
	 *
	 * Результат:
	 * as[j][i] - сплайн коэффициенты Aj ветви i
	 * bs[j][i] - сплайн коэффициенты Bj ветви i
	 * cs[j][i] - сплайн коэффициенты Cj ветви i
	 *
	 * @return true - решение невозможно
	 */
	private final boolean gosub5100(final int i) {
		nt = ni - 1;
		for( int j = 1; j <= ni; j++ ) {// вычисление Ei по уравнению 5.1-19 стр.128 (126)
			final int j1 = j + 1;
			sh[j] = xs[j1][i] - xs[j][i];
			se[j] = (ys[j1][i] - ys[j][i]) / sh[j];
		}
		for( int j = 1; j <= nt; j++ ) {// составление трёхдиагональной системы уравнений 5.1-25 стр. 129 (127)
			final int j1 = j + 1;
			dh[j] = 2 * (sh[j] + sh[j1]);
			dn[j] = sh[j1];
			rs[j] = 3 * (se[j1] - se[j]);
		}
		if( gosub5006( false ) ) {
			return true;
		}
		gosub5026();
		bs[1][i] = 0;
		rs[ni] = 0;
		for( int j = 1; j <= ni; j++ ) {
			bs[j + 1][i] = rs[j];
			as[j][i] = (rs[j] - bs[j][i]) / (3 * sh[j]);
			cs[j][i] = se[j] - sh[j] * (rs[j] + 2 * bs[j][i]) / 3;
		}
		return false;
	}
	/**
	 * Оригинальное название: gosub5200, стр. 134 (132)
	 *
	 * splineb 195 (193)
	 *
	 * Вычисление сплайн-коэффициентов согласно граничному условию б)
	 * B) - крутизна любого интерполирующего сплайна в обеих граничных точках фиксирована.
	 * стр. 126 (124), уравнения 5.1-9
	 *
	 * Вход:
	 * i - номер ветви
	 * ni - количество N интервалов
	 * xs[j][i] - опорные точки Xj
	 * ys[j][i] - значения функции Yj для опорных точек
	 *
	 * Используются:
	 * dh[ni] - элементы главной диагонали
	 * dn[ni] - элементы дополнительной диагонали
	 * nt - количество уравнений
	 * rs[ni] - правая часть (вектор решения)
	 * se[ni] - коэффициенты Ej
	 * sh[ni] - коэффициенты Hj
	 * n1 = nt - 1 FIXME ???
	 * n3 = ni - 3
	 *
	 * Результат:
	 * as[j][i] - сплайн коэффициенты Aj ветви i
	 * bs[j][i] - сплайн коэффициенты Bj ветви i
	 * cs[j][i] - сплайн коэффициенты Cj ветви i
	 *
	 * @return true - решение невозможно
	 */
	private final boolean gosub5200(final int i) {
		final int n1 = ni - 1;// nv FIXME n1 = nt - 1 = ni - 3 or ni - 1?
		nt = ni - 2;// nt
		final int n3 = ni - 3;// n1 = nt - 1
		for( int j = 1; j <= ni; j++ ) {
			final int j1 = j + 1;
			sh[j] = xs[j1][i] - xs[j][i];
			se[j] = (ys[j1][i] - ys[j][i]) / sh[j];
		}
		as[1][i] = 0;
		bs[1][i] = 0;
		cs[1][i] = se[1];
		as[ni][i] = 0;
		bs[ni][i] = 0;
		cs[ni][i] = se[ni];
		for( int j = 1; j <= n3; j++ ) {
			final int j1 = j + 1;
			dh[j] = 2 * (sh[j] + sh[j1]);
			dn[j] = sh[j1];
			rs[j] = 3 * (se[j1] - se[j]);
		}
		dh[1] = 2 * sh[2];
		dh[nt] = 4 * sh[nt] + 3 * sh[n1];
		rs[nt] = 9 * se[n1] - 6 * se[nt] - 3 * se[ni];
		if( gosub5006( true ) ) {// tridiag
			return true;
		}
		// чтобы не выносить r1, следующие 2 операции внесены внутрь gosub5006
		// и их необходимость выбирается аргументом. true - выполнить
		// dh[nt] = dh[nt] - r1 * dn[n3];
		// rs[nt] = rs[nt] - r1 * rs[n3];
		gosub5026();
		cs[2][i] = se[1];
		bs[2][i] = rs[1];
		for( int j = 2; j <= nt; j++ ) {
			final int j1 = j + 1;
			bs[j1][i] = rs[j];
			as[j][i] = (bs[j1][i] - bs[j][i]) / (3 * sh[j]);
			cs[j1][i] = cs[j][i] + sh[j] * (2 * bs[j][i] + 3 * as[j][i] * sh[j]);
		}
		as[n1][i] = ((se[n1] - cs[n1][i]) / sh[n1] - bs[n1][i]) / sh[n1];
		return false;
	}
	/**
	 * Оригинальное название: gosub5300, стр. 136 (134)
	 *
	 * interpol 195 (193)
	 *
	 * Интерполяция
	 *
	 * Вход:
	 * as[j][i], bs[j][i], cs[j][i] - сплайн коэффициенты
	 * i - номер ветви
	 * ns[i] - количество опорных точек
	 * xk - координата x
	 * xs[j][i] - опорные точки
	 * ys[j][i] - значения функции для опорных точек
	 *
	 * Результат:
	 * iv - номер интервала
	 * sk - наклон Mk интерполирующего Xk сплайна для Y
	 * yk - координата Y
	 *
	 * @return true в данной ветви значение xk лежит вне области определения
	 */
	private final boolean gosub5300(final int i) {
		iv = 0;
		if( xk < xs[1][i] ) {// FIXME проверить
			return true;// -> 5316
		}
		for( int i1 = 2; i1 <= ns[i]; i1++ ) {
			if( xk <= xs[i1][i] ) {// бинарный поиск будет быстрее?
				iv = i1 - 1;
				final double x1 = xk - xs[iv][i];
				final double a1 = as[iv][i] * x1;
				yk = ((a1 + bs[iv][i]) * x1 + cs[iv][i]) * x1 + ys[iv][i];
				sk = (3 * a1 + 2 * bs[iv][i]) * x1 + cs[iv][i];
				return false;
			}
		}
		// ветвь I значение XK лежит вне области определения
		System.out.println( CLASS_NAME + ".gosub5300 branch " + i + " XK out of range");
		return true;
	}
	/**
	 * Оригинальное название: gosub5400, стр. 146 (144)
	 *
	 * verteber 194 (192)
	 *
	 * Подготовка интерполяции
	 *
	 * Вход:
	 * i - номер ветви
	 * ns[i] - количество опорных точек
	 * sw[i] - начальное или приблизительное значение
	 * xs[1][i] - опорные точки вблизи нижней границы
	 * xs[n1][i] - опорные точки вблизи верхней границы области определения
	 *
	 * Результат:
	 * xk - координата x для интерполяции
	 * @return xk
	 */
	private final double gosub5400(final int i) {
		final int n1 = ns[i];// количество опорных точек
		if( sw[i] > xs[n1][i] ) {
			xk = xs[n1][i];
			return xk;
		}
		if( sw[i] < xs[1][i] ) {
			xk = xs[1][i];
			return xk;
		}
		xk = sw[i];
		return xk;
		/* verteber 194 (192)
		xk = xs[ ns[i] ][i];
		if( sw[i] < xk ) {
			if( sw[i] < xs[1][i] ) {
				xk = xs[1][i];
			} else {
				xk = sw[i];
			}
		}
		return xk;
		*/
	}
	/**
	 * Оригинальное название: gosub6004, стр. 170 (168)
	 *
	 * Анализ в момент времени t
	 *
	 * @return true - не сходится
	 */
	private final boolean gosub6004(final double[][] result) {
		gosub4000();// FIXME в anazeit этого нет
		// начальное значение каждого нелинейного двухполюсника
		// FIXME! gosub6004 стр. 170 (168), sr[i] = sr[i] + al[j] * zg[i][j] * zg[i][j]
		// anazeit стр. 199 (197), sr[i] = sr[i] + al[j] * zg[i][j] * ze[i][j]
		for( int i = 1; i <= nz; i++ ) {// anazeit 199 (197)
			sr[i] = 0;
			sw[i] = 0;
			for( int j = 1; j <= kb; j++ ) {
				//sr[i] = sr[i] + al[j] * zg[i][j] * zg[i][j];// zg * zg? zg * ze?
				sr[i] = sr[i] + al[j] * zg[i][j] * ze[i][j];
				sw[i] = sw[i] + ga[j] * zg[i][j];
			}
		}
		// uv = un; ?
		// итерация метода Ньютона
		for( int kl = 1; kl <= 50; kl++ ) {// FIXME почему 50?
			// определение дискретной токовой модели для каждого нелинейного двухполюсника
			for( int i = 1; i <= nz; i++ ) {// dis_nlin 196 (194)
				if( ns[i] != 0 ) {// 6034 -> 6048
					xk = sw[i];
					gosub5400( i );
					if( gosub5300( i ) ) {
						return true;
					}
					if( iv != 0 ) {// 6042 -> 6148
						switch( ia[i] ) {
						case 'G':// 1 нелинейная проводимость
							gd[i] = sk;
							sd[i] = yk - sk * xk;
							break;
						case 'R':// 2 нелинейное сопротивление
							gd[i] = 1 / sk;
							sd[i] = xk - yk / sk;
							break;
						case 'C':// 3 нелинейная ёмкость
							gd[i] = -al[0] * yk / h;// FIXME проверить массив al! счет с 0?
							sd[i] = -sr[i] / h;
							C[i] = yk;
							break;
						case 'L':// 4 нелинейная индуктивность
							gd[i] = -h / (al[0] * yk);
							sd[i] = -sr[i] / (al[0] * yk);
							L[i] = yk;
							break;
						}
					}
				} else {
					// управляемая проводимость
					final int j = mg[i];
					gd[i] = sf[i] * (sd[j] + gd[j] * sw[j]);
					sd[i] = 0;
				}
			}// for i 6090 end dis_nlin
			if( gosub4100() ) {// 199 (197) dismod? aufloes ?
				return true;
			}
			// вычисление параметров ветвей нелинейных двухполюсников
			for( int i = 1; i <= nz; i++ ) {
				switch( ia[i] ) {
				case 'G':// 1 нелинейная проводимость
				case 'C':// 3 нелинейная ёмкость
					zs[i] = ua[i];
					break;
				case 'R':// 2 нелинейное сопротивление
				case 'L':// 4 нелинейная индуктивность
					zs[i] = sa[i];
					break;
				}
			}
			// запрос, пригодно ли приближение
			int i = 1;
			for( ; i <= nz; i++ ) {// anazeit 200 (198)
				if( ns[i] != 0 ) {// 6114 -> 6124
					if( Math.abs( sw[i] ) >= 1e-30 ) {// 6116 -> 6122 FIXME 1e-30 как константа
						if( Math.abs( (zs[i] - sw[i]) / sw[i] ) > en ) {// 6118 -> 6130
							break;
						}
					} else if( Math.abs( zs[i] ) > en ) {
						break;
					}
				}
			}// for i 6124
			if( i > nz ) {// цикл завершился сам 172 (170) 6166 umspaus стр. 199 (197)
				// Перезапись параметров состояния и вывод параметров ветви
				for( i = 1; i <= nz; i++ ) {
					for( int j = 11; j >= 2; j-- ) {// FIXME почему от 11 до 2?
						ze[i][j] = ze[i][j - 1];
					}
					switch( ia[i] ) {
					case 'G':// 1 нелинейная проводимость
					case 'R':// 2 нелинейное сопротивление
						break;
					case 'C':// 3 нелинейная ёмкость
						ze[i][1] = C[i];
						break;
					case 'L':// 4 нелинейная индуктивность
						ze[i][1] = L[i];
						break;
					}
				}
				gosub4300( result );
				return false;
			}
			// цикл был прерван
			// новое запоминание параметров ветвей
			for( i = 1; i <= nz; i++ ) {
				if( ns[i] != 0 ) {
					if( kl > 4 ) {
						sw[i] = 0.5 * (zs[i] + sw[i]);
					} else {
						sw[i] = zs[i];
					}
				}
			}
			// 6166
		}// for kl
		System.out.println( CLASS_NAME + ".gosub6004 not converge");
		return true;
	}
	/**
	 * Оригинальное название: sinus, стр. 78 (76)
	 *
	 * sinus 205 (203)
	 *
	 * Анализ линейной цепи синусоидальным напряжением
	 *
	 * введены параметры схемы полосового фильтра рис. 3.12, стр. 86 (84)
	 *
	 * Узловой анализ линейной цепи переменного тока с управляемыми источниками тока.
	 *
	 * f = 743.0 Hz
	 * U4 = 1.862433570933382V/-178.2644293827821 grad
	 * f = 743.2 Hz
	 * U4 = 1.8629162864482216V/-178.85483429453322 grad
	 * f = 743.4 Hz
	 * U4 = 1.8632012204415347V/-179.4453235508272 grad
	 * f = 743.6 Hz
	 * U4 = 1.8632882603873964V/179.9642282019118 grad
	 * f = 743.8 Hz
	 * U4 = 1.8631774827800645V/179.37394626506608 grad
	 *
	 * @param fa начальное значение частоты
	 * @param fe конечное значение частоты
	 * @param fs шаг изменения частоты
	 */
	public final ArrayList<FreqChartData> sinus( final double fa, final double fe, final double fs) {
		final ArrayList<FreqChartData> result = new ArrayList<>();
		//
		//fa = 743;
		//fe = 744;
		//fs = 0.2;
		//
		iz = 8;
		kn = 4;
		is = 1;
		n = kn - is;// кол-во уравнений
		if( iz < is || n <= 0 ) {
			System.out.println( CLASS_NAME + ".sinus incorrect input or solving impossible" );
			return result;
		}
		ir = iz - is;
		/*
		final double[] un = new double[kn];
		final double[] ui = new double[kn];
		final double[] kf = new double[kn];
		final double[][] a = new double[n][n];
		final double[][] b = new double[n][n];
		final double[] x = new double[n];
		final double[] y = new double[n];
		final double[] ka = new double[is];
		final double[] ku = new double[is];
		final double[] qt = new double[is];
		final double[] qi = new double[is];
		*/
		un = new double[kn + 1];
		ui = new double[kn + 1];
		kf = new int[kn + 1];
		//
		a = new double[n + 1][n + 1];
		b = new double[n + 1][n + 1];
		x = new double[n + 1];
		y = new double[n + 1];
		//
		ka = new int[is + 1];
		ku = new int[is + 1];
		qt = new double[is + 1];
		qi = new double[is + 1];
		//
		sf = new double[ir + 1];
		mz = new int[ir + 1];
		vr = new double[ir + 1];
		vi = new double[ir + 1];
		C = new double[ir + 1];
		L = new double[ir + 1];
		//
		kv = new int[ir + 1];
		kz = new int[ir + 1];
		gd = new double[ir + 1];
		gi = new double[ir + 1];
		sd = new double[ir + 1];
		si = new double[ir + 1];
		//
		sa = new double[ir + 1];
		sb = new double[ir + 1];
		ua = new double[ir + 1];
		ub = new double[ir + 1];
		mi = new int[ir + 1];
		mu = new int[ir + 1];
		// ввод структуры цепи
		ig = false;
		//for( int i = 1; i <= iz; i++ ) {
			// ветвь i
			//gosub3500();
			// полосовой фильтр на ОУ, стр. 86 (84)
			// ветвь 1, из узла 1, к узлу 2, элемент R, сопротивление 22000
			gosub3500( 1, 1, 2, 'R', 22000,    0, ' ', 0, 0 );
			// ветвь 2, из узла 4, к узлу 3, элемент R, сопротивление 82000
			gosub3500( 2, 4, 3, 'R', 82000,    0, ' ', 0, 0 );
			// ветвь 3, из узла 2, к узлу 0, элемент R, сопротивление 56
			gosub3500( 3, 2, 0, 'R', 56,    0, ' ', 0, 0 );
			// ветвь 4, из узла 4, к узлу 0, элемент G, сопротивление 0.001,
			// элемент S, параметр U, коэффициент управления -100, управляющая ветвь 5
			gosub3500( 4, 4, 0, 'G', 0.001, 0, ' ', 0, 0 );
			gosub3500( 4, 4, 0, 'S', 0, 0, 'U', -100, 5 );
			// ветвь 5, из узла 0, к узлу 3, элемент G, сопротивление 1e-6
			gosub3500( 5, 0, 3, 'G', 1e-6,    0, ' ', 0, 0 );
			// ветвь 6, из узла 2, к узлу 3, элемент C, ёмкость .1e-6
			gosub3500( 6, 2, 3, 'C', .1e-6,    0, ' ', 0, 0 );
			// ветвь 7, из узла 4, к узлу 2, элемент C, ёмкость .1e-6
			gosub3500( 7, 4, 2, 'C', .1e-6,    0, ' ', 0, 0 );
			// ветвь 8, из узла 1, к узлу 0, источник, напряжение 1, мнимое 0
			gosub3500( 8, 1, 0, ' ', 1, 0,    ' ', 0, 0 );
		//}
		// определение массива узлов
		gosub2100();
		// подготовка перемещения идеальных источников напряжения
		gosub3600();
		// определение выводимых величин
		//gosub3400();// запрос, что надо вывести
		gosub3400( new int[]{4}, new int[0] );
		//}
		// частоты
		for( double f = fa; f <= fe; f += fs ) {
			final double w = 2 * Math.PI * f;
			System.out.println( CLASS_NAME + ".sinus f = " + f + " Hz" );
			// kaufloes 188 (186)
			// составление и решение системы линейных уравнений
			for( int i = 1; i <= n; i++ ) {
				x[i] = 0;
				y[i] = 0;
				for( int j = 1; j <= n; j++ ) {
					a[i][j] = 0;
					b[i][j] = 0;
				}
			}
			for( int i = 1; i <= ir; i++ ) {
				gi[i] = w * C[i];
				if( L[i] != 0 ) {
					gi[i] = gi[i] - 1 / (w * L[i]);
				}
				double s1 = sd[i] + gd[i] * vr[i] - gi[i] * vi[i];
				double s2 = si[i] + gi[i] * vr[i] + gd[i] * vi[i];
				final int j = kv[i];
				final int k = kz[i];
				final int jr = kf[j];
				final int kr = kf[k];
				if( jr != kr ) {// to 332
					a[jr][jr] = a[jr][jr] + gd[i];
					b[jr][jr] = b[jr][jr] + gi[i];
					a[jr][kr] = a[jr][kr] - gd[i];
					b[jr][kr] = b[jr][kr] - gi[i];
					a[kr][jr] = a[kr][jr] - gd[i];
					b[kr][jr] = b[kr][jr] - gi[i];
					a[kr][kr] = a[kr][kr] + gd[i];
					b[kr][kr] = b[kr][kr] + gi[i];
					if( mz[i] != 0 ) {// 276
						final int ms = Math.abs( mz[i] );
						double g1;
						double g2;
						if( mz[i] >= 0 ) {// 280 FIXME kaufloes > 0
							g1 = sf[i];
							g2 = 0;
							s1 = s1 + sf[i] * vr[ms];
							s2 = s2 + sf[i] * vi[ms];
						} else {// 292
							g1 = sf[i] * gd[ms];
							g2 = sf[i] * gi[ms];
							s1 = s1 + g1 * vr[ms] - g2 * vi[ms] + sf[i] * sd[ms];
							s2 = s2 + g2 * vr[ms] + g1 * vi[ms] + sf[i] * si[ms];
						}
						int js = kv[ms];// 300
						int ks = kz[ms];
						js = kf[js];
						ks = kf[ks];
						a[jr][js] = a[jr][js] + g1;
						b[jr][js] = b[jr][js] + g2;
						a[jr][ks] = a[jr][ks] - g1;
						b[jr][ks] = b[jr][ks] - g2;
						a[kr][js] = a[kr][js] - g1;
						b[kr][js] = b[kr][js] - g2;
						a[kr][ks] = a[kr][ks] + g1;
						b[kr][ks] = b[kr][ks] + g2;
					}
					// 324
					x[jr] = x[jr] - s1;
					y[jr] = y[jr] - s2;
					x[kr] = x[kr] + s1;
					y[kr] = y[kr] + s2;
				}// endif
			}// end for i
			ep = 1e-12;
			if( ig ) {// если есть управляемые источники
				if( gosub3300() ) {
					System.out.println( CLASS_NAME + ".sinus solving impossible" );
					return result;
				}
				if( gosub3000() ) {
					System.out.println( CLASS_NAME + ".sinus solving impossible" );
					return result;
				}
			} else {// если управляемых источников нет
				if( gosub3200() ) {
					System.out.println( CLASS_NAME + ".sinus solving impossible" );
					return result;
				}
			}
			// вычисление узловых напряжений
			gosub3800();
			// вычисление и вывод параметров ветвей
			gosub3900();
			if( ma != 0 ) {// 364 -> 382
				final double[] amps = new double[ma];
				final double[] phases = new double[ma];
				for( int i = 1; i <= ma; i++ ) {
					final int j = mu[i];
					//if( j <= ir ) {// 370 -> 380, kausfest стр. 194 (192)
						final double ar = ua[j];// реальная часть комплексной переменной
						final double bi = ub[j];// мнимая часть комплексной переменной
						//gosub3100();
						final double rp = Math.hypot( ar, bi );
						double ph = 90 * Math.signum( bi );
						if( ar != 0 ) {
							ph = 90. * (1 - Math.signum( ar )) * Math.signum( ph + 1. );
							ph = 180. * Math.atan( bi / ar ) / Math.PI + ph;
						}
						amps[i - 1] = rp;
						phases[i - 1] = ph;
						System.out.println( CLASS_NAME + ".sinus U" + j + " = " + rp + "V/" + ph + " grad" );
					//}
				}
				result.add( new FreqChartData( f, amps, phases ) );
			}
			// 382
			if( mb != 0 ) {// 388 -> 414
				final double[] amps = new double[ma];
				final double[] phases = new double[ma];
				for( int i = 1; i <= mb; i++ ) {
					final int j = mi[i];
					//if( j <= ir ) {
						final double ar = sa[j];
						final double bi = sb[j];
						//gosub3100();
						final double rp = Math.hypot( ar, bi );
						double ph = 90 * Math.signum( bi );
						if( ar != 0 ) {
							ph = 90. * (1 - Math.signum( ar )) * Math.signum( ph + 1. );
							ph = 180. * Math.atan( bi / ar ) / Math.PI + ph;
						}
						amps[i - 1] = rp;
						phases[i - 1] = ph;
						System.out.println( CLASS_NAME + ".sinus I" + j + " = " + rp + "V/" + ph + " grad" );
					//}
				}
				result.add( new FreqChartData( f, amps, phases ) );
			}
		}// for f
		return result;
	}
	/**
	 * Оригинальное название: gleich, стр. 57 (55)
	 *
	 * gleich 204 (202)
	 *
	 * Узловой анализ линейной цепи постоянного тока с управляемыми источниками тока
	 *
	 * введены параметры схемы рис. 2.20а, рис.2.23 стр. 58 (56)
	 *
	 * U1 = 10.588180316963951 V; I1 = 1.557085340729993E-4 A
	 * U2 = 9.411819683036049 V; I2 = 1.3840911298582424E-4 A
	 * U3 = 8.764140609296568 V; I3 = 0.005008080348169468 A
	 * U4 = 0.027679073739481197 V; I4 = 1.729942108717575E-5 A
	 * U5 = 6.245078463620855 V; I5 = 0.004990780927082642 A
	 * U6 = 4.990780927082577 V; I6 = 0.004990780927082577 A
	 */
	public final double[][] gleich() {
		iz = 8;// количество ветвей
		kn = 5;// количество узлов (без базового)
		is = 2;// количество идеальных источников напряжения
		n = kn - is;
		if( iz < is || n <= 0 ) {
			System.out.println( CLASS_NAME + ".gleich incorrect input");
			return null;
		}
		ir = iz - is;
		//
		un = new double[kn + 1];
		kf = new int[kn + 1];
		a = new double[n + 1][n + 1];
		x = new double[n + 1];
		ka = new int[is + 1];
		ku = new int[is + 1];
		qt = new double[is + 1];
		//
		sf = new double[ir + 1];
		mz = new int[ir + 1];
		vr = new double[ir + 1];
		//
		kv = new int[ir + 1];
		kz = new int[ir + 1];
		gd = new double[ir + 1];
		sd = new double[ir + 1];
		sa = new double[ir + 1];
		ua = new double[ir + 1];
		//
		// ввод структуры цепи
		ig = false;
		//for( int i = 1; i <= iz; i++ )
		{// ветвь i
			// схема рис. 2.20а, рис.2.23 стр. 58 (56)
			// ветвь 1, из узла 1, к узлу 4, элемент R, сопротивление 68000
			gosub2500( 1, 1, 4, 'R', 68000,    ' ', 0, 0 );
			// ветвь 2, из узла 4, к узлу 0, элемент R, сопротивление 68000
			gosub2500( 2, 4, 0, 'R', 68000,    ' ', 0, 0 );
			// ветвь 3, из узла 3, к узлу 0, элемент R, сопротивление 1750
			gosub2500( 3, 3, 0, 'R', 1750,    ' ', 0, 0 );
			// ветвь 4, из узла 4, к узлу 5, элемент R, сопротивление 1600
			gosub2500( 4, 4, 5, 'R', 1600,    ' ', 0, 0 );
			// ветвь 5, из узла 2, к узлу 3, элемент R, сопротивление 42500
			gosub2500( 5, 2, 3, 'R', 42500,    ' ', 0, 0 );
			// ветвь 5, из узла 2, к узлу 3, элемент S (источник тока), управляющий параметр I, усиление по току 280, управляющая ветвь 4
			gosub2500( 5, 2, 3, 'S', 0, 'I', 280, 4 );
			// ветвь 6, из узла 1, к узлу 2, элемент G, проводимость 0.001 или 0.002
			gosub2500( 6, 1, 2, 'G', 0.001,    ' ', 0, 0 );
			// ветвь 6, из узла 1, к узлу 2, элемент G, проводимость 0.001 или 0.002
			//gosub2500( 6, 1, 2, 'G', 0.002,    ' ', 0, 0 );
			// ветвь 7, из узла 5, к узлу 3, напряжение источник 0.62
			gosub2500( 7, 5, 3, ' ', 0.62,    ' ', 0, 0 );
			// ветвь 8, из узла 1, к узлу 0, напряжение источник 20
			gosub2500( 8, 1, 0, ' ', 20.,    ' ', 0, 0 );
		}
		// определение массива узлов
		gosub2100();
		// подготовка перемещения идеальных источников напряжения
		gosub2600();
		// составление и решение системы линейных уравнений
		if( gosub2700() ) {
			System.out.println( CLASS_NAME + ".gleich not solving" );
			return null;
		}
		// расчёт узловых напряжений
		gosub2800();
		// вычисление и вывод параметров ветвей
		gosub2900();
		for( int i = 1; i <= ir; i++ ) {
			final double eu = ua[i];
			final double ei = sa[i];
			System.out.println( CLASS_NAME + ".gleich U" + i + " = " + eu + " V; I" + i + " = " + ei + " A");
		}
		return null;
	}
	/**
	 * Оригинальное название: dyna-lin, стр. 117 (115)
	 *
	 * Анализ переходных процессов линейной динамической цепи из двухполюсников
	 *
	 * пример схема трансформатора рис. 4.17. стр. 121 (119)
	 *
	 * t = 1.0E-6 s U2 = 0.025604551920341393 V
	 * t = 2.0E-6 s U2 = 0.04982507400715081 V
	 * t = 3.0E-6 s U2 = 0.07273446174503483 V
	 * t = 4.0E-6 s U2 = 0.09440177394044977 V
	 * t = 5.0E-6 s U2 = 0.11489243465223554 V
	 *
	 * t = 6.2E-5 s U2 = 0.441534369844018 V
	 * t = 6.4E-5 s U2 = 0.4419362334829106 V
	 * t = 6.6E-5 s U2 = 0.4421625962803484 V
	 * t = 6.8E-5 s U2 = 0.44223229488204535 V
	 * t = 7.0E-5 s U2 = 0.44216218201946716 V
	 * t = 7.2E-5 s U2 = 0.4419673363701266 V
	 */
	public final double[][] dyna(final double step, final int points) {
		// ввод глобальных параметров
		iz = 6;// количество ветвей
		kn = 4;// количество узлов (без базового)
		is = 1;// количество идеальных источников напряжения
		nz = 0;
		n = kn - is;
		if( iz < is || n <= 0 ) {
			System.out.println( CLASS_NAME + ".dyna incorrect input or solving impossible" );
			return null;
		}
		ir = iz - is;
		//
		un = new double[kn + 1];
		uv = new double[kn + 1];
		kf = new int[kn + 1];
		a = new double[n + 1][n + 1];
		x = new double[n + 1];
		//
		sf = new double[ir + 1];
		mz = new int[ir + 1];
		vr = new double[ir + 1];
		sr = new double[ir + 1];
		//
		kv = new int[ir + 1];
		kz = new int[ir + 1];
		gd = new double[ir + 1];
		sd = new double[ir + 1];
		sa = new double[ir + 1];
		ua = new double[ir + 1];
		//
		G = new double[ir + 1];
		C = new double[ir + 1];
		L = new double[ir + 1];
		mi = new int[ir + 1];
		mu = new int[ir + 1];
		//
		ia = new char[ir + 1];
		zg = new double[ir + 1][11 + 1];
		al = new double[6];
		ga = new double[6];
		//
		it = new char[iz + 1];
		zf = new double[iz + 1];
		vf = new double[iz + 1];
		//
		ka = new int[is + 1];
		ku = new int[is + 1];
		qr = new double[is + 1];
		qt = new double[is + 1];
		////////////////////////////////////
		// ввод структуры цепи
		ig = false;// стр. 117 (115)
		//for( int i = 1; i <= iz; i++ )
		{
			// ветвь i
			//gosub4500( i );
			// ветвь 1, из узла 4, к узлу 3, тип ветви R, сопротивление 50, тип источника N
			gosub4500( 1, 4, 3, 'R', 50, 0, 'N',    ' ', 0, 0, ' ', 0, 0 );
			// ветвь 2, из узла 1, к узлу 0, тип ветви R, сопротивление 50, тип источника N
			gosub4500( 2, 1, 0, 'R', 50, 0, 'N',    ' ', 0, 0, ' ', 0, 0 );
			// ветвь 3, из узла 3, к узлу 2, тип ветви L, индуктивность 0.0009, начальное значение 0
			gosub4500( 3, 3, 2, 'L', 0.0009, 0,   ' ', ' ', 0, 0, ' ', 0, 0 );
			// ветвь 4, из узла 2, к узлу 0, тип ветви L, индуктивность 0.0171, начальное значение 0
			gosub4500( 4, 2, 0, 'L', 0.0171, 0,   ' ', ' ', 0, 0, ' ', 0, 0 );
			// ветвь 5, из узла 2, к узлу 1, тип ветви L, индуктивность 0.009, начальное значение 0
			gosub4500( 5, 2, 1, 'L', 0.0009, 0,   ' ', ' ', 0, 0, ' ', 0, 0 );// опечатка 9mH
			// ветвь 6, из узла 4, к узлу 0, источник напряжения 1, временная зависимость K
			gosub4500( 6, 4, 0, ' ', 1, 0, ' ', 'K', 0, 0, ' ', 0, 0 );
		}
		//////// отсюда начало
		// определение массива узлов
		gosub2100();// 308
		// определение выводимых параметров
		// кол-во напряжений 1, номер ветви 2, кол-во токов 0
		gosub3400( new int[]{2}, new int[0] );
		//}
		// ввод параметров управления
		final double ha = .000001;// начальное значение шага
		final double he = 0.0001;// максимальное значение шага
		final int ke = 5;// максимальное значение порядка
		if( ke < 2 || ke > 6 ) {
			return null;// ввод не имеет смысла
		}
		final double te = 0.001;// конечное значение времени
		// добавлено для вывода результата
		final double[][] result = new double[1][1 + (int)(te / ha)];
		mResultIndex = 0;
		//
		final double eb = 0.01;// значение предела
		// стр. 118 (116) 208
		// При значении шага, меньшем H=HA значение порядка повышается от 1 до KE
		// для каждого значения KB цепь анализируется в MT моментах времени
		final int mt = 10;// конечное значение цикла FIXME почему 10?
		un[0] = 0;
		//en = 0.00001;
		t = 0;
		h = ha;
		jf = false;
		for( kb = 1; kb <= ke; kb++ ) {
			gosub4800();
			for( int ml = 1; ml <= mt; ml++ ) {
				t = t + h;
				gosub4000();
				if( gosub4100() ) {
					System.out.println( CLASS_NAME + ".dyna not converge");
					return null;
				}
				gosub4300( result );
				//if( gosub6004( result ) ) {// FIXME dx, sx не обнулены!
				//	System.out.println( CLASS_NAME + ".dyna not converge");
				//	return null;
				//}
				mResultIndex++;
			}
		}
		kb = ke;
		// для каждого значения MT вычисляется и заданным образом удваивается значение шага
		jf = true;
		do {
			do {
				dx = 0;// 252
				sx = 0;
				for( int ml = 1; ml <= mt; ml++ ) {
					t = t + h;
					if( t > (te + he) ) {// 260 -> 306
						return result;
						//return null;// какая-то проблема
					}
					gosub4000();
					if( gosub4100() ) {
						System.out.println( CLASS_NAME + ".dyna not converge");
						return null;
					}
					gosub4300( result );
					mResultIndex++;
				}
			}
			while( dx / sx > eb );
			h = 2 * h;
			for( int j = 1; j <= 6; j++ ) {
				final int jd = 2 * j - 1;
				for( int i = 1; i <= ir; i++ ) {
					zg[i][j] = zg[i][jd];
				}
			}
		} while( h < he );
		// дальнейшие вычисления выполняются с максимальным значением шага до конечного значения времени
		jf = false;
		do {
			t = t + h;// 438
			gosub4000();
			if( gosub4100() ) {
				System.out.println( CLASS_NAME + ".dyna not converge");
				return null;
			}
			gosub4300( result );
			mResultIndex++;
		} while( t < te );
		// продолжать вычисления? сделать изменения в ветвях и продолжить начиная с 308
		return result;
	}
	/**
	 * Оригинальное название: dyna-nl, стр. 172 (170)
	 *
	 * Анализ переходных процессов динамической цепи с нелинейными двухполюсниками
	 *
	 * пример схема однополупериодного выпрямителя, нагруженного активным опротивлением
	 * рис. 6.8. стр. 176 (174)
	 *
	 * t = 5.0E-5 s U2 = 0.17575505815847892 V
	 * t = 1.0E-4 s U2 = 0.34964095494960634 V
	 * t = 1.5E-4 s U2 = 0.5216317865714277 V
	 * t = 2.0E-4 s U2 = 0.6917019425451508 V
	 * t = 2.5E-4 s U2 = 0.8598261138196631 V
	 * t = 3.0E-4 s U2 = 1.0259793007833757 V
	 * t = 3.5E-4 s U2 = 1.1901368211816052 V
	 * t = 4.0E-4 s U2 = 1.3522743179377208 V
	 *
	 * t = 0.1612 s U2 = 12.609201562475828 V
	 * t = 0.1616 s U2 = 12.836146091014557 V
	 * t = 0.1620 s U2 = 12.960308817964659 V
	 * t = 0.1624 s U2 = 12.971044392015658 V
	 * t = 0.1628 s U2 = 12.865811565443831 V
	 * t = 0.1632 s U2 = 12.697491528304495 V
	 *
	 * t = 0.1812 s U2 = 12.60883169482461 V
	 * t = 0.1816 s U2 = 12.835914982251913 V
	 * t = 0.1820 s U2 = 12.960235093664185 V
	 * t = 0.1824 s U2 = 12.971141585845169 V
	 * t = 0.1828 s U2 = 12.86600710593622 V
	 * t = 0.1832 s U2 = 12.697692281097847 V
	 */
	public final double[][] dynanl(final double step, final int points) {
		// ввод глобальных параметров
		iz = 5;// количество ветвей
		kn = 3;// количество узлов (без базового)
		is = 1;// количество идеальных источников напряжения
		n = kn - is;
		if( iz < is || n <= 0 ) {
			System.out.println( CLASS_NAME + ".dyna incorrect input or solving impossible" );
			return null;
		}
		ir = iz - is;
		//
		un = new double[kn + 1];
		uv = new double[kn + 1];
		kf = new int[kn + 1];
		a = new double[n + 1][n + 1];
		x = new double[n + 1];
		//
		al = new double[6];
		ga = new double[6];
		//
		sf = new double[ir + 1];
		mz = new int[ir + 1];
		vr = new double[ir + 1];
		sr = new double[ir + 1];
		zg = new double[ir + 1][11 + 1];
		//
		kv = new int[ir + 1];
		kz = new int[ir + 1];
		gd = new double[ir + 1];
		sd = new double[ir + 1];
		sa = new double[ir + 1];
		ua = new double[ir + 1];
		//
		G = new double[ir + 1];
		C = new double[ir + 1];
		L = new double[ir + 1];
		mi = new int[ir + 1];
		mu = new int[ir + 1];
		ia = new char[ir + 1];
		//
		it = new char[iz + 1];
		zf = new double[iz + 1];
		vf = new double[iz + 1];
		//
		ka = new int[is + 1];
		ku = new int[is + 1];
		qr = new double[is + 1];
		qt = new double[is + 1];
		//
		// ввод нелинейных двухполюсников
		nz = 1;// количество нелинейных двухполюсников
		//
		mg = new int[nz + 1];
		ns = new int[nz + 1];
		sw = new double[nz + 1];
		ze = new double[nz + 1][11 + 1];
		zs = new double[nz + 1];
		//
		nm = 3;
		{
		// 1 to nz
			final int i = 1;
			// тип двухполюсника G(1)/R(2)/C(3)/L(4)
			ia[i] = 'G';// 1
			//ia[i] = 'R';// 2
			//ia[i] = 'C';// 3
			//ia[i] = 'L';// 4
		//
			ns[i] = 6;// количество опорных точек не менее 3
			if( ns[i] != 0 && ns[i] >= 3 && ns[i] >= nm ) {
				nm = ns[i];
			}
		}
		ni = nm - 1;
		xs = new double[nm + 1][nz + 1];
		ys = new double[nm + 1][nz + 1];
		bs = new double[nm + 1][nz + 1];
		//
		as = new double[ni + 1][nz + 1];
		cs = new double[ni + 1][nz + 1];
		se = new double[ni + 1];
		sh = new double[ni + 1];
		//
		dh = new double[ni + 1];
		dn = new double[ni + 1];
		rs = new double[ni + 1];
		//
		//for( int i = 1; i <= nz; i++ )// 216
		{
			final int i = 1;// ветвь 1
			// System.out.println( CLASS_NAME + ".dyna branch " + i + " from knot " + kv[i] + " to knot " + kz[i] );
			kv[i] = 2;// из узла
			kz[i] = 1;// в узел
			if( ns[i] > 0 ) {// стр. 172, 224 -> 236
				// ввод опорных точек
				ni = ns[i] - 1;
				// двухполюсник i
				// for j = 1 to ns[i]
					int j = 1;
					xs[j][i] = -1000;// x
					ys[j][i] = -1e-5;// y
					j = 2;
					xs[j][i] = 0.6;// x
					ys[j][i] = 0.6e-8;// y
					j = 3;
					xs[j][i] = 0.7;// x
					ys[j][i] = 0.04;// y
					j = 4;
					xs[j][i] = 0.8;// x
					ys[j][i] = 0.3;// y
					j = 5;
					xs[j][i] = 0.9;// x
					ys[j][i] = 1;// y  опечатка 0.1
					j = 6;
					xs[j][i] = 1;// x
					ys[j][i] = 2;// y
				// end for j
				// 254 стр. 174 (172) граничные условия, A,B
				final char condition = 'B';// опечатка а)
				switch( condition ) {
				case 'A':
					if( gosub5100( i ) ) {
						System.out.println( CLASS_NAME + ".dyna not solving" );
						return null;
					}
					break;
				case 'B':
					if( gosub5200( i ) ) {
						System.out.println( CLASS_NAME + ".dyna not solving" );
						return null;
					}
					break;
				default:
					return null;
				}
				// начальное значение стр. 166 (164) п.6.2 для уменьшения вычислительных затрат
				switch( ia[i] ) {
				case 'G':// 1
				case 'C':// 3
					zg[i][1] = 0.7;// напряжение 1 or i?
					break;
				case 'R':// 2
				case 'L':// 4
					zg[i][1] = 0;// ток 1 or i? сюда не попадаем, т.к. тип элемента введен как G
					break;
				}
				switch( ia[i] ) {
				case 'G':// 1
				case 'R':// 2
					break;
				case 'C':// 3
				case 'L':// 4
					xk = zg[i][1];
					if( gosub5300( i ) ) {
						System.out.println( CLASS_NAME + ".dyna not solving" );
						return null;
					}
					if( iv != 0 ) {// если 0 - значения некорректны, надо повторить ввод заново для данного двухполюсника
						ze[i][1] = yk;
					}
					break;
				}
			} else {
				// стр. 173 (171) 236
				// проводимость, управляемая током
				//System.out.println( CLASS_NAME + ".dyna I controlled conduction" );
				//System.out.println( CLASS_NAME + ".dyna Control coefficient " + sf[i] + " controlling branch " + mg[i] + " to knot " + kz[i] );
				sf[i] = 0;// коэффициент управления
				mg[i] = 0;// ветвь управления
			}
		}// 288 завершение цикла ввода данных
		////////////////////////////////////
		// ввод линейных двухполюсников
		ig = false;// стр. 174 (172)
		//for( int i = nz + 1; i <= iz; i++ )
		{
			// ветвь i
			//gosub4500( i );// стр. 110 (108)
			// ветвь 2, из узла 1, к узлу 0, тип ветви C, емкость .47e-3, начальное значение 0
			gosub4500( 2, 1, 0, 'C', .47e-3, 0,    ' ', ' ', 0, 0, ' ', 0, 0 );
			// ветвь 3, из узла 1, к узлу 0, тип ветви R, сопротивление 68, тип источника N
			gosub4500( 3, 1, 0, 'R', 68, 0, 'N',    ' ', 0, 0, ' ', 0, 0 );
			// ветвь 4, из узла 3, к узлу 2, тип ветви R, сопротивление 12, тип источника N
			gosub4500( 4, 3, 2, 'R', 12, 0, 'N',    ' ', 0, 0, ' ', 0, 0 );
			// ветвь 5, из узла 3, к узлу 0, напряжение источника 21, зависимость от времени S, круговая частота 314.15, начальный угол фазы 0
			gosub4500( 5, 3, 0, ' ', 21, 0, ' ', 'S', 314.15, 0,    ' ', 0, 0 );// 2 * pi * 50 Hz
		}
		//////// отсюда начало
		// определение массива узлов
		gosub2100();// 308
		// определение выводимых параметров
		// кол-во напряжений 1, номер ветви 2, кол-во токов 0
		gosub3400( new int[]{2}, new int[0] );
		//}
		// ввод параметров управления
		final double ha = 0.00005;// начальное значение шага
		final int ke = 5;// максимальное значение порядка
		if( ke < 2 || ke > 6 ) {
			return null;// ввод не имеет смысла
		}
		final double te = 0.184;// конечное значение времени
		final double he = 0.0003;// максимальное значение шага
		// добавлено для вывода результата
		final double[][] result = new double[1][1 + (int)(te / ha)];
		mResultIndex = 0;
		/* тест интерполяции кривой диода
		xk = -1000;
		for( int i = 0; xk <= 1; i++ ) {
			gosub5300( 1 );
			result[0][i] = yk;
			xk += 1;
		}
		return result;
		*/
		//
		final double eb = 0.02;// значение предела
		// стр. 174 (172) 348
		// При значении шага, меньшем H=HA значение порядка KB возрастает на 1 до KE
		// для каждого значения KB цепь анализируется в момент времени MT
		final int mt = 10;// конечное значение цикла
		un[0] = 0;
		en = 0.00001;
		t = 0;
		h = ha;
		jf = false;// 0
		for( kb = 1; kb <= ke; kb++ ) {
			gosub4800();// опечатка 4000, koeff
			for( int ml = 1; ml <= mt; ml++ ) {
				t = t + h;
				if( gosub6004( result ) ) {// anazeit FIXME dx, sx не обнулены!
					System.out.println( CLASS_NAME + ".dyna not converge");
					return null;
				}
				mResultIndex++;
			}
		}
		kb = ke;
		// для каждого значения MT рассчитывается и заданным образом удваивается значение шага
		jf = true;// 1
		do {
			do {
				dx = 0;// 392
				sx = 0;
				for( int ml = 1; ml <= mt; ml++ ) {
					t = t + h;
					if( t > (te + he) ) {// 400 -> 446
						return null;// какая-то проблема
					}
					if( gosub6004( result ) ) {// anazeit
						System.out.println( CLASS_NAME + ".dyna not converge");
						return null;
					}
					// ? if( sx > ep ) dx = dx / sx;
					mResultIndex++;
				}
			}
			while( dx / sx > eb );// ? dx < eb
			h = 2 * h;
			for( int j = 1; j <= 6; j++ ) {
				final int jd = 2 * j - 1;
				for( int i = 1; i <= ir; i++ ) {
					zg[i][j] = zg[i][jd];
				}
				for( int i = 1; i <= nz; i++ ) {// ? no in pascal
					ze[i][j] = ze[i][jd];
				}
			}
		} while( h < he );
		// дальнейшие вычисления выполняются с максимальным значением шага вплоть до конечного времени
		jf = false;// 0
		do {
			t = t + h;// 438
			if( gosub6004( result ) ) {// anazeit
				System.out.println( CLASS_NAME + ".dyna not converge");
				return null;
			}
			mResultIndex++;
		} while( t < te );
		// продолжать вычисления? сделать изменения в ветвях и продолжить начиная с 308
		return result;
	}
	/**
	 * Оригинальное название: newton, стр. 152 (150)
	 *
	 * Узловой анализ цепи постоянного тока с нелинейными двухполюсниками
	 *
	 * введены параметры схемы рис. 5.22, стр. 155 (153)
	 *
	 * U1 = 0.7427526478890943 V
	 * U2 = 4.019486444029818 V
	 * U3 = 4.019486444029818 V
	 * U4 = 19.678140301289925 V
	 * U5 = 0.7427526478890943 V
	 * U6 = 16.401406505149204 V
	 * U7 = 3.57910705082098 V
	 * I1 = 1.5170827489681057E-4 A
	 * I2 = 7.622477336413919E-4 A
	 * I3 = 0.03413436185178238 A
	 * I4 = 8.944609227859057E-4 A
	 * I5 = 7.427526478890943E-4 A
	 * I6 = 0.034896609585423836 A
	 * I7 = 0.0357910705082098 A
	 */
	public final void newton() {
		iz = 8;// количество ветвей
		kn = 4;// количество узлов без базового
		is = 1;// количество идеальных источников напряжения
		n = kn - is;
		if( iz < is || n <= 0 ) {
			TheApp.log( CLASS_NAME + ".newton incorrect input");
			return;
		}
		ir = iz - is;
		//
		un = new double[kn + 1];
		kf = new int[kn + 1];
		a = new double[n + 1][n + 1];
		x = new double[n + 1];
		//
		sf = new double[ir + 1];
		mz = new int[ir + 1];
		vr = new double[ir + 1];
		mi = new int[ir + 1];
		mu = new int[ir + 1];
		//
		kv = new int[ir + 1];
		kz = new int[ir + 1];
		gd = new double[ir + 1];
		sd = new double[ir + 1];
		sa = new double[ir + 1];
		ua = new double[ir + 1];
		//
		ka = new int[is + 1];
		ku = new int[is + 1];
		qt = new double[is + 1];
		//
		// ввод нелинейных двухполюсников
		nz = 2;// количество нелинейных двухполюсников
		if( nz < 1 ) {
			TheApp.log( CLASS_NAME + ".newton incorrect input");
			return;
		}
		ia = new char[ir + 1];
		mg = new int[nz + 1];
		ns = new int[nz + 1];
		sw = new double[nz + 1];
		zs = new double[nz + 1];
		nm = 3;
		final char[] twopoles = new char[]{'G','G'};// опечатка, не виден тип двухполюсников
		final int[] basepoints = new int[]{7, 0};
		for( int i = 1; i <= nz; i++ )
		{
			// двухполюсник
			// тип двухполюсника
			ia[i] = twopoles[i - 1];
			ns[i] = basepoints[i - 1];
			if( ns[i] != 0 ) {
				if( ns[i] < 3 && ns[i] < nm ) {// количество опорных точек не менее 3
					TheApp.log( CLASS_NAME + ".newton incorrect input");
					return;
				}
				nm = ns[i];
			}
		}
		ni = nm - 1;
		//
		xs = new double[nm + 1][nz + 1];
		ys = new double[nm + 1][nz + 1];
		bs = new double[nm + 1][nz + 1];
		//
		as = new double[ni + 1][nz + 1];
		cs = new double[ni + 1][nz + 1];
		se = new double[ni + 1];
		sh = new double[ni + 1];
		//
		dh = new double[ni + 1];
		dn = new double[ni + 1];
		rs = new double[ni + 1];
		//
		//for( int i = 1; i <= nz; i++ )
		{
			// у второго 0 точек
			int i = 1;// ветвь i
			kv[i] = 2;// из узла
			kz[i] = 0;// к узлу
			if( ns[i] <= 0 ) {
				// проводимость, управляемая током
				sf[i] = 0;// коэффициент управления
				mg[i] = 0;// ветвь управления
			} else {
				// ввод опорных точек
				ni = ns[i] - 1;
				// двухполюсник i
				//for( int j = 1; j <= ns[i]; j++ )
				{
					int j = 1;
					xs[j][i] = 0;// X
					ys[j][i] = 0;// Y
					j = 2;
					xs[j][i] = 0.55;// X
					ys[j][i] = 0;// Y
					j = 3;
					xs[j][i] = 0.6;// X
					ys[j][i] = 5e-6;// Y
					j = 4;
					xs[j][i] = 0.65;// X
					ys[j][i] = 20e-6;// Y
					j = 5;
					xs[j][i] = 0.7;// X
					ys[j][i] = 75e-6;// Y
					j = 6;
					xs[j][i] = 0.75;// X
					ys[j][i] = 180e-6;// Y
					j = 7;
					xs[j][i] = 0.8;// X
					ys[j][i] = 400e-6;// Y
				}
				// граничные условия A/B
				final char condition = 'B';
				switch( condition ) {
				default:
					TheApp.log( CLASS_NAME + ".newton incorrect input");
					return;
				case 'A':
					if( gosub5100( i ) ) {
						System.out.println( CLASS_NAME + ".newton not solving" );
						return;
					}
					break;
				case 'B':
					if( gosub5200( i ) ) {
						System.out.println( CLASS_NAME + ".newton not solving" );
						return;
					}
					break;
				}
				// начальное значение
				switch( ia[i] ) {
				default:
					TheApp.log( CLASS_NAME + ".newton incorrect input");
					return;
				case 'G':
					// напряжения
					sw[i] = 0.75;
					break;
				case 'R':
					// тока
					sw[i] = 0;
					break;
				}
			}// if else
			i = 2;// ветвь i
			kv[i] = 3;// из узла
			kz[i] = 0;// к узлу
			if( ns[i] <= 0 ) {
				// проводимость, управляемая током
				sf[i] = 1.25;// коэффициент управления
				mg[i] = 1;// ветвь управления
			}// if else
		}// for i
		// ввод линейных двухполюсников
		ig = false;
		//for( int i = nz + 1; i <= iz; i++ )
		{
			// ветвь 3, из узла 3, к узлу 0, элемент S, управляемый источник тока, управляющая величина I, усиление по току 225, управляющая ветвь 1
			gosub2500( 3, 3, 0, 'S', 0, 'I', 225, 1 );
			// ветвь 4, из узла 1, к узлу 2, элемент R, сопротивление 22000, элемент N
			gosub2500( 4, 1, 2, 'R', 22000,    ' ', 0, 0 );
			// ветвь 5, из узла 2, к узлу 0, элемент R, сопротивление 1000, элемент N
			gosub2500( 5, 2, 0, 'R', 1000,    ' ', 0, 0 );
			// ветвь 6, из узла 1, к узлу 3, элемент R, сопротивление 470, элемент N
			gosub2500( 6, 1, 3, 'R', 470,    ' ', 0, 0 );
			// ветвь 7, из узла 4, к узлу 1, элемент R, сопротивление 100, элемент N
			gosub2500( 7, 4, 1, 'R', 100,    ' ', 0, 0 );
			// ветвь 8, из узла 4, к узлу 0, напряжение источника 24
			gosub2500( 8, 4, 0, ' ', 24,    ' ', 0, 0 );
		}
		// определение выводимых величин
		gosub3400( new int[]{1,2,3,4,5,6,7}, new int[]{1,2,3,4,5,6,7} );
		// определение массива узлов
		gosub2100();
		// подготовка перемещения идеальных источников напряжения
		gosub2600();
		// итерация метода Ньютона
		en = 0.00001;
		// сюда закольцовано при расходимости и вводе новых параметров ветви
		int kl = 1;
		for( ; kl <= 20; kl++ ) {// 300, стр. 154 (152)
			// определение дискретной токовой модели для каждого нелинейного двухполюсника
			for( int i = 1; i <= nz; i++ ) {
				if( ns[i] != 0 ) {
					gosub5400( i );
					if( gosub5300( i ) ) {
						TheApp.log( CLASS_NAME + ".newton not converge");
						return;
					}
					if( iv != 0 ) {
						switch( ia[i] ) {
						case 'G':
							// нелинейная проводимость
							gd[i] = sk;
							sd[i] = yk - sk * xk;
							break;
						case 'R':
							gd[i] = 1. / sk;
							sd[i] = xk - yk / sk;
							break;
						}
					} else {
						TheApp.log( CLASS_NAME + ".newton need new values");
						return;
					}
				} else {
					// управляемая проводимость
					final int j = mg[i];
					gd[i] = sf[i] * (sd[j] + gd[j] * sw[j]);
					sd[i] = 0;
				}
			}
			// составление и решение системы линейных уравнений
			gosub2700();
			// расчет узловых напряжений
			gosub2800();
			// вычисление параметров ветвей нелинейных двухполюсников
			for( int i = 1; i <= nz; i++ ) {
				final int j = kv[i];
				final int k = kz[i];
				ua[i] = un[j] - un[k];
				sa[i] = ua[i] * gd[i] + sd[i];
				switch( ia[i] ) {
				case 'G':
					zs[i] = ua[i];
					break;
				case 'R':
					zs[i] = sa[i];
					break;
				}
			}
			// запрос, пригодно ли приближение
			int i = 1;
			for( ; i <= nz; i++ ) {
				if( ns[i] != 0 ) {
					if( Math.abs( sw[i] ) < 1e-30 ) {// 388 -> 394 стр. 154 (150)
						if( Math.abs( zs[i] ) > en ) {// 394 -> 402
							break;
						}
					} else {
						if( Math.abs( (zs[i] - sw[i]) / sw[i] ) > en ) {
							break;
						}
					}
				}
			}// for i, 396
			if( i > nz ) {// цикл завершился сам
				// 434
				// вывод параметров ветви
				gosub2900();
				gosub4326( null );
				break;// ввести новое начальное значение
			}
			// цикл был прерван
			// запоминание параметров ветвей
			for( i = 1; i <= nz; i++ ) {
				if( ns[i] != 0 ) {
					if( kl <= 4 ) {
						sw[i] = zs[i];
					} else {
						sw[i] = .5 * (zs[i] + sw[i]);
					}
				}
			}
		}// for kl
		if( kl > 20 ) {
			// прерывание ввиду расходимости
			TheApp.log( CLASS_NAME + ".newton interrupted, divergence");
		}
		// выбрать новое начальное значение
		TheApp.log( CLASS_NAME + ".newton type new start value");
	}
	/**
	 * Оригинальное название: spline, стр. 137 (135)
	 *
	 * Интерполяция с помощью сплайн функций
	 *
	 * введены параметры рис. 5.6 стр. 138 (136)
	 *
	 * extremum: -4.803847577293368 ; -2.3982241950953678
	 * extremum: 0.0 ; 0.9999999999999999
	 * extremum: 0.0 ; 0.9999999999999999
	 * extremum: 4.803847577293368 ; -2.3982241950953678
	 */
	public final void spline() {
		final int i = 1;
		nm = 5;// количество опорных точек
		//
		xs = new double[nm + 1][1 + 1];
		ys = new double[nm + 1][1 + 1];
		bs = new double[nm + 1][1 + 1];
		ns = new int[1 + 1];
		//
		ns[i] = nm;
		ni = nm - 1;
		//
		as = new double[ni + 1][1 + 1];
		cs = new double[ni + 1][1 + 1];
		se = new double[ni + 1];
		sh = new double[ni + 1];
		dh = new double[ni + 1];
		dn = new double[ni + 1];
		rs = new double[ni + 1];
		//
		// ввод опорных точек
		final double[] xp = new double[]{-10, -1, 0, 1, 10};
		final double[] yp = new double[]{  0,  0, 1, 0,  0};
		for( int j = 1; j <= ns[i]; j++ )
		{
			xs[j][i] = xp[j - 1];// X[j] =
			ys[j][i] = yp[j - 1];// Y[j] =
		}
		// граничные условия, A/B
		final char condition = 'A';
		switch( condition ) {// расчет сплайн коэффициентов
		default:
			return;
		case 'A':
			if( gosub5100( i ) ) {
				System.out.println( CLASS_NAME + ".spline not solving" );
				return;
			}
			break;
		case 'B':
			if( gosub5200( i ) ) {
				System.out.println( CLASS_NAME + ".spline not solving" );
				return;
			}
			break;
		}
		for( int j = 1; j <= ni; j++ ) {
			if( as[j][i] != 0 ) {// стр. 137 (135) 158 -> 178
				// корни квадратного уравнения 3 a x^2 + 2 b x + c = 0
				final double w1 = Math.pow( bs[j][i], 2 ) - 3 * as[j][i] * cs[j][i];
				if( w1 >= 0 ) {
					if( w1 > 0 ) {
						final double x1 = (-bs[j][i] + Math.sqrt( w1 )) / (3 * as[j][i]);
						gosub200( i, j, x1 );
					}
					final double x1 = (-bs[j][i] - Math.sqrt( w1 )) / (3 * as[j][i]);
					gosub200( i, j, x1 );
				}
			} else {
				// 178
				if( bs[j][i] != 0 ) {// a = 0, 2 b x + c = 0
					final double x1 = -cs[j][i] / (2 * bs[j][i]);
					gosub200( i, j, x1 );
				}
			}
		}// for j 184
		int output_counter = 0;
		xk = -1.;// вместо ввода увеличиваем в цикле
		do {
			do {
				// интерполяция, запрос X, 190
				//xk = 0;// x =
				if( gosub5300( i ) ) {
					System.out.println( CLASS_NAME + ".spline not solving" );
					return;
				}
				xk += 0.1;
			} while( iv == 0 );
			System.out.println( CLASS_NAME + ".spline y = " + yk );
			output_counter++;
		} while( output_counter < 20 );
	}
	/**
	 * вычисление экстремума
	 * @param i
	 * @param j
	 * @param x1
	 */
	private final void gosub200(final int i, final int j, final double x1) {
		xk = x1 + xs[j][i];
		if( xk <= xs[j + 1][i] ) {
			if( xk >= xs[j][i] ) {// опечатка xk <= xs[j + 1][i]?
				if( gosub5300( i ) ) {
					System.out.println( CLASS_NAME + ".gosub200 interpolation impossible" );
					return;
				}
				System.out.println( CLASS_NAME + ".gosub200 extremum: " + xk + " ; " + yk );
			}
		}
	}
	/**
	 * Точка входа.
	 * @param args "sinus" - Анализ линейной цепи синусоидальным напряжением,<br>
	 * "gleich" - Узловой анализ линейной цепи постоянного тока с управляемыми источниками тока,<br>
	 * "dyna" - Анализ переходных процессов линейной динамической цепи из двухполюсников,<br>
	 * "dyna-nl" - Анализ переходных процессов динамической цепи с нелинейными двухполюсниками,<br>
	 * "newton" - Узловой анализ цепи постоянного тока с нелинейными двухполюсниками,<br>
	 * "spline" - Интерполяция с помощью сплайн функций
	 */
	public static void main(final String[] args) {
		if( args.length == 0 ) {
			System.out.println( CLASS_NAME + ".main no argument [sinus, gleich, dyna, dyna-nl, newton, spline], quit");
			return;
		}
		switch( args[0].toLowerCase() ) {
		case "sinus":
			new BasicAnalyzerOriginal().sinus( 743, 744, 0.2 );
			break;
		case "gleich":
			new BasicAnalyzerOriginal().gleich();
			break;
		case "dyna":
			new BasicAnalyzerOriginal().dyna( 0, 0 );
			break;
		case "dyna-nl":
			new BasicAnalyzerOriginal().dynanl( 0, 0 );
			break;
		case "newton":
			new BasicAnalyzerOriginal().newton();
			break;
		case "spline":
			new BasicAnalyzerOriginal().spline();
			break;
		}
	}
}
